@@ -13,10 +13,10 @@
 =========================================================================*/
 /*
  * HISTORY:
- * In GDCM 1.X the prefered terms was 'ReWrite', however one author of GDCM dislike
+ * In GDCM 1.X the preferred term was 'ReWrite', however one author of GDCM dislike
  * the term ReWrite since it is associated with the highly associated with the Rewrite
  * notion in software programming where using reinvent the wheel and rewrite from scratch code
- * the term convert was prefered
+ * the term convert was preferred
  *
  * Tools to conv. Goals being to 'purify' a DICOM file.
  * For now it will do the minimum:
@@ -25,7 +25,7 @@
  *   simply discarded (not written).
  * - Elements are written in alphabetical order
  * - 32bits VR have the residue bytes sets to 0x0,0x0
- * - Same goes from Item Length end delimitor, sets to 0x0,0x0
+ * - Same goes from Item Length end delimiter, sets to 0x0,0x0
  * - All buggy files (wrong length: GE, 13 and Siemens Leonardo) are fixed
  * - All size are even (no odd length from gdcm 1.x)
  *
@@ -44,7 +44,7 @@
  * - Any broken JPEG file (wrong bits) should be fixed
  * - DicomObject bug should be fixed
  * - Meta and Dataset should have a matching UID (more generally File Meta
- *   should be correct (Explicit!) and consistant with DataSet)
+ *   should be correct (Explicit!) and consistent with DataSet)
  * - User should be able to specify he wants Group Length (or remove them)
  * - Media SOP should be correct (deduct from something else or set to
  *   SOP Secondary if all else fail).
@@ -80,6 +80,7 @@
 #include "gdcmImage.h"
 #include "gdcmImageChangeTransferSyntax.h"
 #include "gdcmImageApplyLookupTable.h"
+#include "gdcmFileDecompressLookupTable.h"
 #include "gdcmImageFragmentSplitter.h"
 #include "gdcmImageChangePlanarConfiguration.h"
 #include "gdcmImageChangePhotometricInterpretation.h"
@@ -104,14 +105,14 @@ struct SetSQToUndefined
   }
 };
 
-void PrintVersion()
+static void PrintVersion()
 {
   std::cout << "gdcmconv: gdcm " << gdcm::Version::GetVersion() << " ";
   const char date[] = "$Date$";
   std::cout << date << std::endl;
 }
 
-void PrintLossyWarning()
+static void PrintLossyWarning()
 {
   std::cout << "You have selected a lossy compression transfer syntax." << std::endl;
   std::cout << "This will degrade the quality of your input image, and can." << std::endl;
@@ -120,7 +121,7 @@ void PrintLossyWarning()
   std::cout << "WARNING: this mode is very experimental." << std::endl;
 }
 
-void PrintHelp()
+static void PrintHelp()
 {
   PrintVersion();
   std::cout << "Usage: gdcmconv [OPTION] input.dcm output.dcm" << std::endl;
@@ -140,6 +141,8 @@ void PrintHelp()
   std::cout << "     --remove-retired      Remove retired tags." << std::endl;
   std::cout << "Image only Options:" << std::endl;
   std::cout << "  -l --apply-lut                      Apply LUT (non-standard, advanced user only)." << std::endl;
+  std::cout << "  -8 --apply-lut8                     Apply LUT/RGB8 (non-standard, advanced user only)." << std::endl;
+  std::cout << "     --decompress-lut                 Decompress LUT (linearize segmented LUT)." << std::endl;
   std::cout << "  -P --photometric-interpretation %s  Change Photometric Interpretation (when possible)." << std::endl;
   std::cout << "  -w --raw                            Decompress image." << std::endl;
   std::cout << "  -d --deflated                       Compress using deflated (gzip)." << std::endl;
@@ -150,7 +153,7 @@ void PrintHelp()
   std::cout << "  -F --force                          Force decompression/merging before recompression/splitting." << std::endl;
   std::cout << "     --generate-icon                  Generate icon." << std::endl;
   std::cout << "     --icon-minmax %d,%d              Min/Max value for icon." << std::endl;
-  std::cout << "     --icon-auto-minmax               Automatically commpute best Min/Max values for icon." << std::endl;
+  std::cout << "     --icon-auto-minmax               Automatically compute best Min/Max values for icon." << std::endl;
   std::cout << "     --compress-icon                  Decide whether icon follows main TransferSyntax or remains uncompressed." << std::endl;
   std::cout << "     --planar-configuration [01]      Change planar configuration." << std::endl;
   std::cout << "  -Y --lossy                          Use the lossy (if possible) compressor." << std::endl;
@@ -166,7 +169,7 @@ void PrintHelp()
   std::cout << "JPEG Options:" << std::endl;
   std::cout << "  -q --quality %*f           set quality." << std::endl;
   std::cout << "JPEG-LS Options:" << std::endl;
-  std::cout << "  -e --lossy-error %*i             set error." << std::endl;
+  std::cout << "  -e --allowed-error %*i             set allowed error." << std::endl;
   std::cout << "J2K Options:" << std::endl;
   std::cout << "  -r --rate    %*f           set rate." << std::endl;
   std::cout << "  -q --quality %*f           set quality." << std::endl;
@@ -186,7 +189,7 @@ void PrintHelp()
 }
 
 template <typename T>
-size_t readvector(std::vector<T> &v, const char *str)
+static size_t readvector(std::vector<T> &v, const char *str)
 {
   if( !str ) return 0;
   std::istringstream os( str );
@@ -422,7 +425,7 @@ static bool derives( File & file, const Pixmap& compressed_image )
 return true;
 
 }
-}
+} // end namespace gdcm
 
 int main (int argc, char *argv[])
 {
@@ -436,6 +439,8 @@ int main (int argc, char *argv[])
   int implicit = 0;
   int quiet = 0;
   int lut = 0;
+  int lut8 = 0;
+  int decompress_lut = 0;
   int raw = 0;
   int deflated = 0;
   int rootuid = 0;
@@ -483,28 +488,28 @@ int main (int argc, char *argv[])
   int jpeglserror = 0;
   int jpeglserror_value = 0;
 
-  while (1) {
+  while (true) {
     //int this_option_optind = optind ? optind : 1;
     int option_index = 0;
     static struct option long_options[] = {
-        {"input", 1, 0, 0},
-        {"output", 1, 0, 0},
-        {"group-length", 1, 0, 0}, // valid / create / remove
-        {"preamble", 1, 0, 0}, // valid / create / remove
-        {"padding", 1, 0, 0}, // valid (\0 -> space) / optimize (at most 1 byte of padding)
-        {"vr", 1, 0, 0}, // valid
-        {"sop", 1, 0, 0}, // default to SC...
-        {"iod", 1, 0, 0}, // valid
-        {"meta", 1, 0, 0}, // valid / create / remove
-        {"dataset", 1, 0, 0}, // valid / create / remove?
-        {"sequence", 1, 0, 0}, // defined / undefined
-        {"deflate", 1, 0, 0}, // 1 - 9 / best = 9 / fast = 1
-        {"tag", 1, 0, 0}, // need to specify a tag xxxx,yyyy = value to override default
-        {"name", 1, 0, 0}, // same as tag but explicit use of name
+        {"input", 1, nullptr, 0},
+        {"output", 1, nullptr, 0},
+        {"group-length", 1, nullptr, 0}, // valid / create / remove
+        {"preamble", 1, nullptr, 0}, // valid / create / remove
+        {"padding", 1, nullptr, 0}, // valid (\0 -> space) / optimize (at most 1 byte of padding)
+        {"vr", 1, nullptr, 0}, // valid
+        {"sop", 1, nullptr, 0}, // default to SC...
+        {"iod", 1, nullptr, 0}, // valid
+        {"meta", 1, nullptr, 0}, // valid / create / remove
+        {"dataset", 1, nullptr, 0}, // valid / create / remove?
+        {"sequence", 1, nullptr, 0}, // defined / undefined
+        {"deflate", 1, nullptr, 0}, // 1 - 9 / best = 9 / fast = 1
+        {"tag", 1, nullptr, 0}, // need to specify a tag xxxx,yyyy = value to override default
+        {"name", 1, nullptr, 0}, // same as tag but explicit use of name
         {"root-uid", 1, &rootuid, 1}, // specific Root (not GDCM)
         {"check-meta", 0, &checkmeta, 1}, // specific Root (not GDCM)
 // Image specific options:
-        {"pixeldata", 1, 0, 0}, // valid
+        {"pixeldata", 1, nullptr, 0}, // valid
         {"apply-lut", 0, &lut, 1}, // default (implicit VR, LE) / Explicit LE / Explicit BE
         {"raw", 0, &raw, 1}, // default (implicit VR, LE) / Explicit LE / Explicit BE
         {"deflated", 0, &deflated, 1}, // DeflatedExplicitVRLittleEndian
@@ -514,8 +519,8 @@ int main (int argc, char *argv[])
         {"jpegls", 0, &jpegls, 1}, // JPEG-LS: lossy / lossless
         {"j2k", 0, &j2k, 1}, // J2K: lossy / lossless
         {"rle", 0, &rle, 1}, // lossless !
-        {"mpeg2", 0, 0, 0}, // lossy !
-        {"jpip", 0, 0, 0}, // ??
+        {"mpeg2", 0, nullptr, 0}, // lossy !
+        {"jpip", 0, nullptr, 0}, // ??
         {"split", 1, &split, 1}, // split fragments
         {"planar-configuration", 1, &planarconf, 1}, // Planar Configuration
         {"explicit", 0, &explicitts, 1}, //
@@ -530,6 +535,8 @@ int main (int argc, char *argv[])
         {"remove-retired", 0, &removeretired, 1}, //
         {"photometric-interpretation", 1, &photometricinterpretation, 1}, //
         {"with-private-dict", 0, &changeprivatetags, 1}, //
+        {"decompress-lut", 0, &decompress_lut, 1}, // linearized segmented LUT
+        {"apply-lut8", 0, &lut8, 1},
 // j2k :
         {"rate", 1, &rate, 1}, //
         {"quality", 1, &quality, 1}, // will also work for regular jpeg compressor
@@ -548,10 +555,10 @@ int main (int argc, char *argv[])
         {"ignore-errors", 0, &ignoreerrors, 1},
         {"quiet", 0, &quiet, 1},
 
-        {0, 0, 0, 0}
+        {nullptr, 0, nullptr, 0}
     };
 
-    c = getopt_long (argc, argv, "i:o:XMUClwdJKLRFYS:P:VWDEhvIr:q:t:n:e:",
+    c = getopt_long (argc, argv, "i:o:XMUCl8wdJKLRFYS:P:VWDEhvIr:q:t:n:e:",
       long_options, &option_index);
     if (c == -1)
       {
@@ -562,7 +569,7 @@ int main (int argc, char *argv[])
       {
     case 0:
         {
-        const char *s = long_options[option_index].name;
+        const char *s = long_options[option_index].name; (void)s;
         //printf ("option %s", s);
         if (optarg)
           {
@@ -617,7 +624,7 @@ int main (int argc, char *argv[])
             {
             assert( strcmp(s, "tile") == 0 );
             size_t n = readvector(tilesize, optarg);
-            assert( n == 2 );
+            assert( n == 2 ); (void)n;
             }
           else if( option_index == 45 ) /* number of resolution */
             {
@@ -669,6 +676,11 @@ int main (int argc, char *argv[])
       lut = 1;
       break;
 
+    case '8':
+      lut8 = 1;
+      break;
+
+
     case 'w':
       raw = 1;
       break;
@@ -781,7 +793,7 @@ int main (int argc, char *argv[])
     while (optind < argc)
       {
       //printf ("%s\n", argv[optind++]);
-      files.push_back( argv[optind++] );
+      files.emplace_back(argv[optind++] );
       }
     //printf ("\n");
     if( files.size() == 2
@@ -914,7 +926,7 @@ int main (int argc, char *argv[])
     }
 
   // Handle here the general file (not required to be image)
-  if ( explicitts || implicit || deflated )
+  if ( !raw && (explicitts || implicit || deflated) )
     {
     if( explicitts && implicit ) return 1; // guard
     if( explicitts && deflated ) return 1; // guard
@@ -941,7 +953,9 @@ int main (int argc, char *argv[])
     gdcm::FileMetaInformation &fmi = file.GetHeader();
 
     const gdcm::TransferSyntax &orits = fmi.GetDataSetTransferSyntax();
-    if( orits != gdcm::TransferSyntax::ExplicitVRLittleEndian && orits != gdcm::TransferSyntax::ImplicitVRLittleEndian && orits != gdcm::TransferSyntax::DeflatedExplicitVRLittleEndian )
+    if( orits != gdcm::TransferSyntax::ExplicitVRLittleEndian
+      && orits != gdcm::TransferSyntax::ImplicitVRLittleEndian
+      && orits != gdcm::TransferSyntax::DeflatedExplicitVRLittleEndian )
       {
       std::cerr << "Sorry input Transfer Syntax not supported for this conversion: " << orits << std::endl;
       return 1;
@@ -956,10 +970,13 @@ int main (int argc, char *argv[])
       {
       ts = gdcm::TransferSyntax::DeflatedExplicitVRLittleEndian;
       }
-    const char *tsuid = gdcm::TransferSyntax::GetTSString( ts );
-    // const char * is ok since padding is \0 anyway...
+    std::string tsuid = gdcm::TransferSyntax::GetTSString( ts );
+    if( tsuid.size() % 2 == 1 )
+      {
+      tsuid.push_back( 0 ); // 0 padding
+      }
     gdcm::DataElement de( gdcm::Tag(0x0002,0x0010) );
-    de.SetByteValue( tsuid, (uint32_t)strlen(tsuid) );
+    de.SetByteValue( tsuid.data(), (uint32_t)tsuid.size() );
     de.SetVR( gdcm::Attribute<0x0002, 0x0010>::GetVR() );
     fmi.Clear();
     fmi.Replace( de );
@@ -1058,7 +1075,37 @@ int main (int argc, char *argv[])
       return 1;
       }
     }
-  else if( lut )
+  else if( decompress_lut )
+  {
+    gdcm::PixmapReader reader;
+    reader.SetFileName( filename.c_str() );
+    if( !reader.Read() )
+      {
+      std::cerr << "Could not read (pixmap): " << filename << std::endl;
+      return 1;
+      }
+    const gdcm::Pixmap &image = reader.GetPixmap();
+
+    gdcm::FileDecompressLookupTable lutfilt;
+    lutfilt.SetFile( reader.GetFile() );
+    lutfilt.SetPixmap( image );
+    bool b = lutfilt.Change();
+    if( !b )
+      {
+      std::cerr << "Could not decompress LUT: " << filename << std::endl;
+      return 1;
+      }
+    gdcm::PixmapWriter writer;
+    writer.SetFileName( outfilename.c_str() );
+    writer.SetFile( reader.GetFile() );
+    writer.SetPixmap( lutfilt.GetPixmap() );
+    if( !writer.Write() )
+      {
+      std::cerr << "Failed to write: " << outfilename << std::endl;
+      return 1;
+      }
+  }
+  else if( lut || lut8 )
     {
     gdcm::PixmapReader reader;
     reader.SetFileName( filename.c_str() );
@@ -1071,6 +1118,7 @@ int main (int argc, char *argv[])
 
     gdcm::ImageApplyLookupTable lutfilt;
     lutfilt.SetInput( image );
+    lutfilt.SetRGB8( lut8 != 0 );
     bool b = lutfilt.Apply();
     if( !b )
       {
@@ -1113,7 +1161,7 @@ int main (int argc, char *argv[])
         {
         iig.SetPixelMinMax( iconmin, iconmax );
         }
-      iig.AutoPixelMinMax( iconautominmax );
+      iig.AutoPixelMinMax( iconautominmax ? true : false );
       bool b = iig.Generate();
       if( !b ) return 1;
       const gdcm::IconImage &icon = iig.GetIconImage();
@@ -1130,12 +1178,16 @@ int main (int argc, char *argv[])
       {
       if( lossy )
         {
-        change.SetTransferSyntax( gdcm::TransferSyntax::JPEGBaselineProcess1 );
+        const gdcm::PixelFormat &pf = image.GetPixelFormat();
+        if( pf.GetBitsAllocated() > 8 )
+          change.SetTransferSyntax(gdcm::TransferSyntax::JPEGExtendedProcess2_4);
+        else
+          change.SetTransferSyntax( gdcm::TransferSyntax::JPEGBaselineProcess1 );
         jpegcodec.SetLossless( false );
         if( quality )
           {
           assert( qualities.size() == 1 );
-          jpegcodec.SetQuality( qualities[0] );
+          jpegcodec.SetQuality( static_cast<double>(qualities[0]) );
           }
         change.SetUserCodec( &jpegcodec );
         }
@@ -1171,7 +1223,7 @@ int main (int argc, char *argv[])
           int i = 0;
           for(std::vector<float>::const_iterator it = rates.begin(); it != rates.end(); ++it )
             {
-            j2kcodec.SetRate(i++, *it );
+            j2kcodec.SetRate(i++, static_cast<double>(*it) );
             }
           }
         if( quality )
@@ -1179,7 +1231,7 @@ int main (int argc, char *argv[])
           int i = 0;
           for(std::vector<float>::const_iterator it = qualities.begin(); it != qualities.end(); ++it )
             {
-            j2kcodec.SetQuality( i++, *it );
+            j2kcodec.SetQuality( i++, static_cast<double>(*it) );
             }
           }
         if( tile )
@@ -1213,11 +1265,15 @@ int main (int argc, char *argv[])
       if( ts.IsExplicit() )
         {
         change.SetTransferSyntax( gdcm::TransferSyntax::ExplicitVRLittleEndian );
+        if( implicit )
+          change.SetTransferSyntax( gdcm::TransferSyntax::ImplicitVRLittleEndian );
         }
       else
         {
         assert( ts.IsImplicit() );
         change.SetTransferSyntax( gdcm::TransferSyntax::ImplicitVRLittleEndian );
+        if( explicitts )
+        change.SetTransferSyntax( gdcm::TransferSyntax::ExplicitVRLittleEndian );
         }
 #endif
       }
@@ -1275,7 +1331,7 @@ int main (int argc, char *argv[])
     if( lossy )
       {
       if(!quiet)
-      PrintLossyWarning();
+        PrintLossyWarning();
       if( !gdcm::derives( reader.GetFile(), change.PixmapToPixmapFilter::GetOutput() ) )
         {
         std::cerr << "Failed to derives: " << filename << std::endl;
@@ -1356,7 +1412,7 @@ int main (int argc, char *argv[])
     std::vector<char> buffer;
     buffer.resize(len); // black image
 
-    ir.GetBuffer( &buffer[0] );
+    ir.GetBuffer( buffer.data() );
     gdcm::ByteValue *bv = new gdcm::ByteValue(buffer);
     gdcm::DataElement pixeldata( gdcm::Tag(0x7fe0,0x0010) );
     pixeldata.SetValue( *bv );
