@@ -19,7 +19,7 @@
  * - dcmInfo (SIEMENS)
  * - PrintFile (GDCM 1.x)
  *
- * For now all layout are harcoded (see --color/--xml-dict for instance)
+ * For now all layout are hardcoded (see --color/--xml-dict for instance)
  *
  * gdcmdump has some feature not described in the DICOM standard:
  *   --csa : to print CSA information (dcmInfo.exe compatible)
@@ -47,18 +47,27 @@
 #include "gdcmPDBHeader.h"
 #include "gdcmSequenceOfItems.h"
 #include "gdcmASN1.h"
+#include "gdcmAttribute.h"
+#include "gdcmBase64.h"
+#include "gdcmMEC_MR3.h"
+#include "gdcmTagKeywords.h"
+#include "gdcmDeflateStream.h"
 
 #include <string>
 #include <iostream>
+#ifdef GDCM_HAVE_CODECVT
+#include <codecvt>
+#endif
+#include <locale>
 
 #include <stdio.h>     /* for printf */
 #include <stdlib.h>    /* for exit */
 #include <getopt.h>
 #include <string.h>
 
-int color = 0;
+static int color = 0;
 
-int ignoreerrors = 0;
+static int ignoreerrors = 0;
 
 namespace cleanup
 {
@@ -71,7 +80,7 @@ enum {
 };
 
 template <typename T>
-void printvaluet(std::istream & is, uint32_t numels)
+static void printvaluet(std::istream & is, uint32_t numels)
 {
   T buffer;
   for( uint32_t i = 0; i < numels; ++i )
@@ -82,7 +91,7 @@ void printvaluet(std::istream & is, uint32_t numels)
     }
 }
 
-void printvalue(std::istream &is, uint32_t type, uint32_t numels, uint32_t pos)
+static void printvalue(std::istream &is, uint32_t type, uint32_t numels, uint32_t pos)
 {
   assert( numels > 0 );
   std::streampos start = is.tellg();
@@ -130,24 +139,67 @@ private:
   }
 };
 
-void printbinary(std::istream &is, PDFElement const & pdfel )
+static void printbinary(std::istream &is, PDFElement const & pdfel )
 {
   const char *bufferref = pdfel.getname();
   std::cout << "  " << bufferref << " ";
   uint32_t type = pdfel.gettype();
   uint32_t numels = pdfel.getnumelems();
-  uint32_t dummy = pdfel.getdummy();
-  assert( dummy == 0 );
+  //uint32_t dummy = pdfel.getdummy();
+  //assert( dummy == 0 ); (void)dummy;
   uint32_t offset = pdfel.getoffset();
   uint32_t pos = (uint32_t)(offset + is.tellg() - 4);
   printvalue(is, type, numels, pos);
 }
 
-void ProcessSDSData( std::istream & is )
+static void ProcessSDSDataInt( std::istream & is )
 {
-  // havent been able to figure out what was the begin meant for
+  (void)is;
+  std::cerr << "TODO" << std::endl;
+}
+
+// see read_direct_string / read_direct_int
+static void ProcessSDSDataString( std::istream & is )
+{
+  (void)is;
+#if 0
+  int32_t v1;
+  is.read( (char*)&v1,sizeof(v1));
+  assert( v1 == 0x1 );
+  uint32_t bla;
+  is.read( (char*)&bla, sizeof(bla) );
+  char name0[32];
+  memset(name0,0,sizeof(name0));
+  assert( bla < sizeof(name0) );
+  is.read( name0, bla);
+  size_t l = strlen(name0);
+  assert( l == bla );
+  std::cerr << "name0:" << name0 << std::endl;
+
+  is.read( (char*)&bla, sizeof(bla) );
+  assert( bla == 0x1 );
+  is.read( (char*)&bla, sizeof(bla) );
+  char value[32];
+  memset(value,0,sizeof(value));
+  assert( bla < sizeof(value) );
+  is.read( value, bla);
+  is.read( (char*)&bla, sizeof(bla) );
+  assert( bla == 0 ); // trailing stuff ?
+  is.read( (char*)&bla, sizeof(bla) );
+  assert( bla == 0 ); // trailing stuff ?
+  const uint32_t cur = (uint32_t)is.tellg();
+  std::cerr << "offset:" << cur << std::endl;
+#else
+  std::cerr << "TODO" << std::endl;
+#endif
+}
+
+static void ProcessSDSData( std::istream & is )
+{
+  // haven't been able to figure out what was the begin meant for
   is.seekg( 0x20 - 8 );
   uint32_t version = 0;
+  assert( sizeof(uint32_t) == 4 );
   is.read( (char*)&version, sizeof(version) );
   assert( version == 8 );
   uint32_t numel = 0;
@@ -166,7 +218,7 @@ void ProcessSDSData( std::istream & is )
 
 }
 // PMS MR Series Data Storage
-int DumpPMS_MRSDS(const gdcm::DataSet & ds)
+static int DumpPMS_MRSDS(const gdcm::DataSet & ds)
 {
   const gdcm::PrivateTag tdata(0x2005,0x32,"Philips MR Imaging DD 002");
   if( !ds.FindDataElement( tdata ) ) return 1;
@@ -211,9 +263,142 @@ int DumpPMS_MRSDS(const gdcm::DataSet & ds)
       {
       s4 = std::string( bv4->GetPointer(), bv4->GetLength() );
       }
-    std::istringstream is( s3 );
     std::cout << "PMS/Item name: [" << s1 << "/" << s2 << "/" << s4 << "]" << std::endl;
-    ProcessSDSData( is );
+    if( bv3 ) {
+      std::istringstream is( s3 );
+      char v1 = is.peek();
+      if( v1 == 0x1 ) {
+        if( s1 == "HARDWARE_CONFIG " )
+          ProcessSDSDataInt( is );
+        else if( s1 == "COILSTATE " )
+          ProcessSDSDataString( is );
+        else
+          assert(0);
+      } else {
+        ProcessSDSData( is );
+      }
+    } else {
+      std::cout << " EMPTY !" << std::endl;
+    }
+    }
+  return 0;
+}
+
+static int DumpTOSHIBA_MEC_CT3(const gdcm::DataSet & ds)
+{
+  const gdcm::PrivateTag tdata(0x7005,0x10,"TOSHIBA_MEC_CT3");
+  if( !ds.FindDataElement( tdata ) ) return 1;
+  const gdcm::DataElement &data = ds.GetDataElement( tdata );
+
+  const gdcm::ByteValue *bv = data.GetByteValue();
+  if( !bv ) return 1;
+
+  const unsigned int offset = 24;
+  if( bv->GetLength() < offset )
+    {
+    std::cerr << "Not enough header" << std::endl;
+    return 1;
+    }
+  std::istringstream is0;
+  const std::string str0 = std::string( bv->GetPointer(), offset );
+  is0.str( str0 );
+  gdcm::ImplicitDataElement ide0;
+  ide0.Read<gdcm::SwapperNoOp>(is0);
+  gdcm::ImplicitDataElement ide1;
+  ide1.Read<gdcm::SwapperNoOp>(is0);
+
+  gdcm::Attribute<0x0,0x0> at0;
+  at0.SetFromDataElement( ide0 );
+  if( at0.GetValue() != 12 )
+    {
+    std::cerr << "Bogus header value #0" << std::endl;
+    return 1;
+    }
+  gdcm::Attribute<0x0,0x1> at1;
+  at1.SetFromDataElement( ide1 );
+
+  const unsigned int dlen = bv->GetLength() - offset;
+  if( at1.GetValue() != dlen )
+    {
+    std::cerr << "Bogus header value #1" << std::endl;
+    return 1;
+    }
+  std::istringstream is1;
+  const std::string str1 = std::string( bv->GetPointer() + offset, bv->GetLength() - offset);
+  is1.str( str1 );
+
+  gdcm::Reader r;
+  r.SetStream( is1 );
+  if( !r.Read() )
+    {
+    std::cerr << "Could not read CT Private Data 2" << std::endl;
+    return 1;
+    }
+
+  gdcm::Printer printer;
+  printer.SetFile ( r.GetFile() );
+  printer.SetColor( color != 0 );
+  printer.Print( std::cout );
+
+  return 0;
+}
+
+static bool DumpToshibaDTI( const char * input, size_t len )
+{
+  if( len % 2 ) return false;
+
+  std::vector<char> copy( input, input + len );
+  std::reverse( copy.begin(), copy.end() );
+
+  std::istringstream is;
+  std::string dup( copy.data(), copy.size() );
+  is.str( dup );
+
+  gdcm::File file;
+  gdcm::FileMetaInformation & fmi = file.GetHeader();
+  fmi.SetDataSetTransferSyntax( gdcm::TransferSyntax::ExplicitVRLittleEndian );
+  gdcm::DataSet & ds = file.GetDataSet();
+  ds.Read<gdcm::ExplicitDataElement,gdcm::SwapperNoOp>( is );
+
+  gdcm::Printer p;
+  //gdcm::DictPrinter p;
+  p.SetFile( file );
+  p.SetColor( color != 0 );
+  p.Print( std::cout );
+
+  return true;
+}
+
+
+static int DumpTOSHIBA_Reverse(const gdcm::DataSet & ds, const gdcm::PrivateTag &tpmtf, const gdcm::PrivateTag &tseq)
+{
+  // (0029,0010) ?? (LO) [PMTF INFORMATION DATA ]                      # 22,1 Private Creator
+  // (0029,1001) ?? (SQ) (Sequence with undefined length)              # u/l,1 ?
+
+  if( !ds.FindDataElement( tpmtf) ) return 0; // this is not an error at this point
+  const gdcm::DataElement& pmtf = ds.GetDataElement( tpmtf );
+  if ( pmtf.IsEmpty() ) return 1;
+  gdcm::SmartPointer<gdcm::SequenceOfItems> seq = pmtf.GetValueAsSQ();
+  if ( !seq || !seq->GetNumberOfItems() ) return 1;
+
+  size_t n = seq->GetNumberOfItems();
+  for( size_t i = 1; i <= n; ++i )
+    {
+    std::cout << "Item #" << i << std::endl;
+    gdcm::Item &item = seq->GetItem(i);
+    gdcm::DataSet &subds = item.GetNestedDataSet();
+    // (0029,0010) ?? (LO) [PMTF INFORMATION DATA ]                  # 22,1 Private Creator
+    // (0029,1090) ?? (OB) 00\05\00\13\00\12\00\22\                  # 202,1 ?
+
+    if( subds.FindDataElement( tseq ) )
+      {
+      const gdcm::DataElement & de = subds.GetDataElement( tseq );
+      const gdcm::ByteValue * bv = de.GetByteValue();
+      if( !bv ) return 1;
+
+      bool b = DumpToshibaDTI( bv->GetPointer(), bv->GetLength() );
+      if( !b ) return 1;
+      }
     }
   return 0;
 }
@@ -311,7 +496,7 @@ struct Data2
    os << "  UserAdress4: " << std::string(UserAdress4,32) << "\n";
    os << "  UserAdress5: " << std::string(UserAdress5,32) << "\n";
    os << "  RecDate: "     << std::string(RecDate,8) << "\n";
-   os << "  RecTime: "     << std::string(RecTime,64) << "\n";
+   os << "  RecTime: "     << std::string(RecTime,6) << "\n";
    os << "  RecPlace: "    << std::string(RecPlace,64) << "\n";
    os << "  RecSource: "   << std::string(RecSource,64) << "\n";
    os << "  DF1: "         << std::string(DF1,64) << "\n";
@@ -341,11 +526,11 @@ struct Data2
    }
 };
 
-bool ProcessData( const char *buf, size_t len )
+static bool ProcessData( const char *buf, size_t len )
 {
   Data2 data2;
-  size_t s = sizeof(data2);
-  assert( len >= s);
+  const size_t s = sizeof(data2);
+  assert( len >= s); (void)len;
   // VIMDATA2 is generally 2048 bytes, while s = 1786
   // the end is filled with \0 bytes
   memcpy(&data2, buf, s);
@@ -354,10 +539,10 @@ bool ProcessData( const char *buf, size_t len )
   return true;
 }
 
-int DumpVEPRO(const gdcm::DataSet & ds)
+static int DumpVEPRO(const gdcm::DataSet & ds)
 {
   // 01f7,1026
-  const gdcm::ByteValue *bv2 = NULL;
+  const gdcm::ByteValue *bv2 = nullptr;
   const gdcm::PrivateTag tdata1(0x55,0x0020,"VEPRO VIF 3.0 DATA");
   const gdcm::PrivateTag tdata2(0x55,0x0020,"VEPRO VIM 5.0 DATA");
   // Prefer VIF over VIM ?
@@ -384,7 +569,7 @@ int DumpVEPRO(const gdcm::DataSet & ds)
 }
 
 // ELSCINT1
-bool readastring(std::string &out, const char *input )
+static bool readastring(std::string &out, const char *input )
 {
   out.clear();
   while( *input )
@@ -508,7 +693,7 @@ struct info
     }
 };
 
-int DumpEl2_new(const gdcm::DataSet & ds)
+static int DumpEl2_new(const gdcm::DataSet & ds)
 {
   // 01f7,1026
   const gdcm::PrivateTag t01f7_26(0x01f7,0x1026,"ELSCINT1");
@@ -518,6 +703,26 @@ int DumpEl2_new(const gdcm::DataSet & ds)
   const gdcm::ByteValue * bv = de01f7_26.GetByteValue();
 
   const char *begin = bv->GetPointer();
+  const size_t buf_len= bv->GetLength();
+
+  bool isgzip = false;
+  if( buf_len > (0x15f + 3) ) {
+    const unsigned char *sig = (const unsigned char*)begin + 0x15f;
+    isgzip = sig[0] == 0x1f && sig[1] == 0x8b && sig[2] == 0x08 /* DEFLATE */;
+  }
+
+  if( isgzip ) {
+    size_t offset = 0x15f;
+    size_t len = buf_len - 0x15f;
+    std::string str( begin + offset, begin + len );
+    std::istringstream is( str );
+  
+    zlib_stream::zip_istream gzis( is );
+    std::string out;
+    while( std::getline(gzis, out) ) {
+      std::cout << out << std::endl;
+    }
+  } else {
   uint32_t val0[3];
   memcpy( &val0, begin, sizeof( val0 ) );
   assert( val0[0] == 0xF22D );
@@ -537,6 +742,7 @@ int DumpEl2_new(const gdcm::DataSet & ds)
     std::cout << std::endl;
     begin += o;
     }
+  }
 
   return 0;
 }
@@ -544,7 +750,7 @@ int DumpEl2_new(const gdcm::DataSet & ds)
 } // end namespace cleanup
 
 template <typename TPrinter>
-int DoOperation(const std::string & filename)
+static int DoOperation(const std::string & filename)
 {
   gdcm::Reader reader;
   reader.SetFileName( filename.c_str() );
@@ -564,7 +770,7 @@ int DoOperation(const std::string & filename)
   return success ? 0 : 1;
 }
 
-int PrintASN1(const std::string & filename, bool verbose)
+static int PrintASN1(const std::string & filename, bool verbose)
 {
   (void)verbose;
   gdcm::Reader reader;
@@ -603,7 +809,7 @@ int PrintASN1(const std::string & filename, bool verbose)
   return 0;
 }
 
-int PrintELSCINT(const std::string & filename, bool verbose)
+static int PrintELSCINT(const std::string & filename, bool verbose)
 {
   (void)verbose;
   gdcm::Reader reader;
@@ -620,7 +826,7 @@ int PrintELSCINT(const std::string & filename, bool verbose)
   return ret;
 }
 
-int PrintVEPRO(const std::string & filename, bool verbose)
+static int PrintVEPRO(const std::string & filename, bool verbose)
 {
   (void)verbose;
   gdcm::Reader reader;
@@ -637,7 +843,7 @@ int PrintVEPRO(const std::string & filename, bool verbose)
   return ret;
 }
 
-int PrintSDS(const std::string & filename, bool verbose)
+static int PrintSDS(const std::string & filename, bool verbose)
 {
   (void)verbose;
   gdcm::Reader reader;
@@ -654,8 +860,154 @@ int PrintSDS(const std::string & filename, bool verbose)
   return ret;
 }
 
+static int PrintCT3(const std::string & filename, bool verbose)
+{
+  (void)verbose;
+  gdcm::Reader reader;
+  reader.SetFileName( filename.c_str() );
+  if( !reader.Read() )
+    {
+    std::cerr << "Failed to read: " << filename << std::endl;
+    return 1;
+    }
 
-int PrintPDB(const std::string & filename, bool verbose)
+  const gdcm::DataSet& ds = reader.GetFile().GetDataSet();
+  int ret = cleanup::DumpTOSHIBA_MEC_CT3( ds );
+
+  return ret;
+}
+
+static int PrintMEC_MR3(const std::string & filename, bool verbose)
+{
+  (void)verbose;
+  gdcm::Reader reader;
+  reader.SetFileName( filename.c_str() );
+  if( !reader.Read() )
+    {
+    std::cerr << "Failed to read: " << filename << std::endl;
+    return 1;
+    }
+
+  const gdcm::DataSet& ds = reader.GetFile().GetDataSet();
+
+  // Original Data
+  const gdcm::PrivateTag tmecmr3(0x700d,0x08,"TOSHIBA_MEC_MR3");
+  if( !ds.FindDataElement( tmecmr3) ) return 1;
+  const gdcm::DataElement& mecmr3 = ds.GetDataElement( tmecmr3 );
+  if ( mecmr3.IsEmpty() ) return 1;
+  const gdcm::ByteValue * bv = mecmr3.GetByteValue();
+
+  const char *inbuffer = bv->GetPointer();
+  const size_t buf_len= bv->GetLength();
+
+  int ret = 0;
+  if (!gdcm::MEC_MR3::Print(inbuffer, buf_len))
+  {
+    ret = 1;
+  }
+
+  return ret;
+}
+
+static int PrintPMTF(const std::string & filename, bool verbose)
+{
+  (void)verbose;
+  gdcm::Reader reader;
+  reader.SetFileName( filename.c_str() );
+  if( !reader.Read() )
+    {
+    std::cerr << "Failed to read: " << filename << std::endl;
+    return 1;
+    }
+
+  const gdcm::DataSet& ds = reader.GetFile().GetDataSet();
+  int ret = 0;
+  {
+  const gdcm::PrivateTag tpmtf(0x0029,0x1,"PMTF INFORMATION DATA");
+  static const gdcm::PrivateTag &tseq = gdcm::MEC_MR3::GetPMTFInformationDataTag();
+  ret += cleanup::DumpTOSHIBA_Reverse( ds, tpmtf, tseq );
+  }
+
+  {
+  const gdcm::PrivateTag tpmtf(0x0029,0x1,"CANON_MEC_MR3");
+  static const gdcm::PrivateTag &tseq = gdcm::MEC_MR3::GetCanonMECMR3Tag();
+  ret += cleanup::DumpTOSHIBA_Reverse( ds, tpmtf, tseq );
+
+  // PAS Reproduct Information
+  const gdcm::PrivateTag tpasri(0x700d,0x19,"CANON_MEC_MR3^10");
+  if( ds.FindDataElement( tpasri) ) {
+  const gdcm::DataElement& pasri = ds.GetDataElement( tpasri );
+  if (! pasri.IsEmpty() ) {
+  const gdcm::ByteValue * bv = pasri.GetByteValue();
+  std::string s(bv->GetPointer(), bv->GetLength() );
+  std::cout << std::endl;
+  std::cout << "PAS Reproduct Information (XML)" << std::endl;
+  // header states: 
+  // <?xml version="1.0" encoding="UTF-8"?>
+  // so simply dump without further cleanup:
+  std::cout << s.c_str() << std::endl;
+  }
+  }
+  }
+
+  {
+  const gdcm::PrivateTag tpmtf(0x0029,0x1,"TOSHIBA_MEC_MR3");
+  static const gdcm::PrivateTag &tseq = gdcm::MEC_MR3::GetToshibaMECMR3Tag();
+  ret += cleanup::DumpTOSHIBA_Reverse( ds, tpmtf, tseq );
+
+  const gdcm::PrivateTag tpmtf2(0x0029,0x2,"TOSHIBA_MEC_MR3");
+  ret += cleanup::DumpTOSHIBA_Reverse( ds, tpmtf2, tseq );
+  }
+
+  return ret;
+}
+
+static void print_utf8(std::string const& s)
+{
+  const char delim = 0;
+  auto start = 0U;
+  auto end = s.find(delim);
+  while (end != std::string::npos)
+  {
+      std::cout << s.substr(start, end - start) << std::endl;
+      start = end + 1;
+      end = s.find(delim, start);
+  }
+}
+
+static int PrintMedComHistory(const std::string & filename, bool verbose)
+{
+  (void)verbose;
+  gdcm::Reader reader;
+  reader.SetFileName( filename.c_str() );
+  if( !reader.Read() )
+    {
+    std::cerr << "Failed to read: " << filename << std::endl;
+    return 1;
+    }
+
+  const gdcm::DataSet& ds = reader.GetFile().GetDataSet();
+  const gdcm::PrivateTag tmedcom(0x0029,0x20,"SIEMENS MEDCOM HEADER");
+  if( !ds.FindDataElement( tmedcom) ) return 1;
+  const gdcm::DataElement& medcom = ds.GetDataElement( tmedcom);
+  if ( medcom.IsEmpty() ) return 1;
+  const gdcm::ByteValue * bv = medcom.GetByteValue();
+
+  const size_t size = bv->GetLength();
+  std::u16string u16((size / 2) + 0, '\0');
+  bv->GetBuffer( (char*)u16.data(), size );
+
+#ifdef GDCM_HAVE_CODECVT
+  std::string utf8 = std::wstring_convert<
+    std::codecvt_utf8_utf16<char16_t>, char16_t>{}.to_bytes(u16);
+  print_utf8(utf8);
+#else
+  std::cerr << "Missing GDCM_HAVE_CODECVT support" << std::endl;
+#endif
+  return 0;
+}
+
+static int PrintPDB(const std::string & filename, bool verbose)
 {
   (void)verbose;
   gdcm::Reader reader;
@@ -681,14 +1033,90 @@ int PrintPDB(const std::string & filename, bool verbose)
     }
   if( !found )
     {
-    std::cout << "no pdb tag found" << std::endl;
+    std::cerr << "no pdb tag found" << std::endl;
     ret = 1;
     }
 
   return ret;
 }
 
-int PrintCSA(const std::string & filename)
+static int PrintCSABase64Impl(gdcm::CSAHeader &csa, std::string const & csaname )
+{
+  const char *name = csaname.c_str();
+  if( csa.FindCSAElementByName(name) )
+  {
+    const gdcm::CSAElement & el = csa.GetCSAElementByName(name);
+    // this is not error if empty:
+    if( el.IsEmpty() ) return 0;
+    const gdcm::ByteValue* bv = el.GetByteValue();
+    std::string str( bv->GetPointer(), bv->GetLength() );
+    str.erase(std::remove(str.begin(), str.end(), '\n'), str.end());
+    size_t dl = gdcm::Base64::GetDecodeLength( str.c_str(), str.size() );
+    std::vector<char> buf;
+    buf.resize( dl );
+    size_t dl2 = gdcm::Base64::Decode( buf.data(), buf.size(), str.c_str(), str.size() );
+    if( dl != dl2 ) return 1;
+    std::stringstream ss;
+    ss.str( std::string(buf.data(), buf.size()) );
+    gdcm::File file;
+    gdcm::DataSet &ds2 = file.GetDataSet();
+    gdcm::DataElement xde;
+    try
+    {
+      while( xde.Read<gdcm::ExplicitDataElement,gdcm::SwapperNoOp>( ss ) )
+      {
+        ds2.Insert( xde );
+      }
+      assert( ss.eof() );
+    }
+    catch(std::exception &)
+    {
+      return 1;
+    }
+    gdcm::Printer p;
+    //gdcm::DictPrinter p;
+    p.SetFile( file );
+    //p.SetColor( 1 );
+    std::cout << "--" << csaname << "--" << std::endl;
+    p.Print(std::cout);
+  }
+  return 0;
+}
+
+static int PrintCSABase64(const std::string & filename, std::vector<std::string> const & csanames )
+{
+  gdcm::Reader reader;
+  reader.SetFileName( filename.c_str() );
+  if( !reader.Read() )
+  {
+    std::cerr << "Failed to read: " << filename << std::endl;
+    return 1;
+  }
+  const gdcm::DataSet& ds = reader.GetFile().GetDataSet();
+
+  int ret = 0;
+
+  gdcm::CSAHeader csa;
+  const gdcm::PrivateTag &t1 = csa.GetCSAImageHeaderInfoTag();
+  if( !ds.FindDataElement( t1 ) ) return 1;
+  csa.LoadFromDataElement( ds.GetDataElement( t1 ) );
+  for( std::vector<std::string>::const_iterator it = csanames.begin();
+		  it != csanames.end(); ++it ) {
+    ret += PrintCSABase64Impl(csa, *it );
+  }
+
+  const gdcm::PrivateTag &t2 = csa.GetCSASeriesHeaderInfoTag();
+  if( !ds.FindDataElement( t2 ) ) return 1;
+  csa.LoadFromDataElement( ds.GetDataElement( t2 ) );
+  for( std::vector<std::string>::const_iterator it = csanames.begin();
+		  it != csanames.end(); ++it ) {
+    ret += PrintCSABase64Impl(csa, *it );
+  }
+
+  return ret;
+}
+
+static int PrintCSA(const std::string & filename)
 {
   gdcm::Reader reader;
   reader.SetFileName( filename.c_str() );
@@ -714,7 +1142,7 @@ int PrintCSA(const std::string & filename)
     found = true;
     if( csa.GetFormat() == gdcm::CSAHeader::ZEROED_OUT )
       {
-      std::cout << "CSA Header has been zero-out (contains only 0)" << std::endl;
+      std::cerr << "CSA Header has been zero-out (contains only 0)" << std::endl;
       ret = 1;
       }
     else if( csa.GetFormat() == gdcm::CSAHeader::DATASET_FORMAT )
@@ -733,7 +1161,7 @@ int PrintCSA(const std::string & filename)
     found = true;
     if( csa.GetFormat() == gdcm::CSAHeader::ZEROED_OUT )
       {
-      std::cout << "CSA Header has been zero-out (contains only 0)" << std::endl;
+      std::cerr << "CSA Header has been zero-out (contains only 0)" << std::endl;
       ret = 1;
       }
     else if( csa.GetFormat() == gdcm::CSAHeader::DATASET_FORMAT )
@@ -752,7 +1180,7 @@ int PrintCSA(const std::string & filename)
     found = true;
     if( csa.GetFormat() == gdcm::CSAHeader::ZEROED_OUT )
       {
-      std::cout << "CSA Header has been zero-out (contains only 0)" << std::endl;
+      std::cerr << "CSA Header has been zero-out (contains only 0)" << std::endl;
       ret = 1;
       }
     else if( csa.GetFormat() == gdcm::CSAHeader::INTERFILE )
@@ -771,23 +1199,87 @@ int PrintCSA(const std::string & filename)
     }
   if( !found )
     {
-    std::cout << "no csa tag found" << std::endl;
+    std::cerr << "no csa tag found" << std::endl;
     ret = 1;
     }
 
   return ret;
 }
 
+static int PrintMrProtocol(const std::string & filename)
+{
+  gdcm::Reader reader;
+  reader.SetFileName( filename.c_str() );
+  if( !reader.Read() )
+  {
+    std::cerr << "Failed to read: " << filename << std::endl;
+    return 1;
+  }
+
+  gdcm::CSAHeader csa;
+  const gdcm::DataSet& ds = reader.GetFile().GetDataSet();
+  gdcm::MrProtocol mrprot;
+  const gdcm::PrivateTag att1(0x21,0x19,"SIEMENS MR SDS 01");
+  const gdcm::PrivateTag att2(0x21,0xfe,"SIEMENS MR SDS 01");
+  bool found = false;
+  namespace kwd = gdcm::Keywords;
+  kwd::SharedFunctionalGroupsSequence sfgs;
+  (void)sfgs;
+  if( csa.GetMrProtocol(ds, mrprot))
+  {
+    found = true;
+  }
+  else if( ds.FindDataElement( att1) )
+  {
+    const gdcm::DataElement &data = ds.GetDataElement( att1 );
+    const gdcm::ByteValue *bv = data.GetByteValue();
+    static const char csastr[] = "PhoenixMetaProtocol"; // FIXME
+    if( mrprot.Load( bv, csastr, -1))
+      found = true;
+  }
+  // SIEMENS now supports Enhanced MR
+  else if( ds.FindDataElement( sfgs.GetTag() ) )
+  {
+    const gdcm::DataElement &shared = ds.GetDataElement( sfgs.GetTag() );
+    gdcm::SmartPointer<gdcm::SequenceOfItems> sqi = shared.GetValueAsSQ();
+    if( sqi != nullptr && sqi->GetNumberOfItems() == 1 ) {
+      gdcm::Item &item = sqi->GetItem(1);
+      gdcm::DataSet & subds = item.GetNestedDataSet();
+      if( subds.FindDataElement( att2) ) {
+        const gdcm::DataElement &privsq = subds.GetDataElement( att2 );
+        gdcm::SmartPointer<gdcm::SequenceOfItems> sqi2 = privsq.GetValueAsSQ();
+        if( sqi2 != nullptr && sqi2->GetNumberOfItems() == 1 ) {
+          gdcm::Item &item2 = sqi2->GetItem(1);
+          gdcm::DataSet & subds2 = item2.GetNestedDataSet();
+          if( subds2.FindDataElement( att1) ) {
+            const gdcm::DataElement &data = subds2.GetDataElement( att1 );
+            const gdcm::ByteValue *bv = data.GetByteValue();
+            static const char csastr[] = "PhoenixMetaProtocol"; // FIXME
+            if( mrprot.Load( bv, csastr, -1))
+              found = true;
+          }
+        }
+      }
+    }
+  }
+
+  if( found )
+    std::cout << mrprot;
+  else
+    std::cerr << "Could not find MrProtocol/MrPhoenixProtocol ASCII section" << std::endl;
+
+  return found ? 0 : 1;
+}
 
 
-void PrintVersion()
+static void PrintVersion()
 {
   std::cout << "gdcmdump: gdcm " << gdcm::Version::GetVersion() << " ";
   const char date[] = "$Date$";
   std::cout << date << std::endl;
 }
 
-void PrintHelp()
+static void PrintHelp()
 {
   PrintVersion();
   std::cout << "Usage: gdcmdump [OPTION]... FILE..." << std::endl;
@@ -801,11 +1293,22 @@ void PrintHelp()
   std::cout << "  -p --print          print value instead of simply dumping (default)." << std::endl;
   std::cout << "  -c --color          print in color." << std::endl;
   std::cout << "  -C --csa            print SIEMENS CSA Header (0029,[12]0,SIEMENS CSA HEADER)." << std::endl;
+  std::cout << "     --csa-asl        print decoded SIEMENS CSA MR_ASL (base64)." << std::endl;
+  std::cout << "     --csa-diffusion  print decoded SIEMENS CSA MRDiffusion (base64)." << std::endl;
+  std::cout << "     --csa64          print decoded SIEMENS CSA FMRI/MR_ASL/MRDiffusion... (base64)." << std::endl;
+  std::cout << "     --mrprotocol     print SIEMENS CSA MrProtocol only (within ASCCONV BEGIN/END)." << std::endl;
+  std::cout << "                      or Phoenix Meta Protocol (0021,19,SIEMENS MR SDS 01)." << std::endl;
   std::cout << "  -P --pdb            print GEMS Protocol Data Block (0025,1b,GEMS_SERS_01)." << std::endl;
   std::cout << "     --elscint        print ELSCINT Protocol Information (01f7,26,ELSCINT1)." << std::endl;
   std::cout << "     --vepro          print VEPRO Protocol Information (0055,20,VEPRO VIF 3.0 DATA)." << std::endl;
   std::cout << "                         or VEPRO Protocol Information (0055,20,VEPRO VIM 5.0 DATA)." << std::endl;
   std::cout << "     --sds            print Philips MR Series Data Storage (1.3.46.670589.11.0.0.12.2) Information (2005,32,Philips MR Imaging DD 002)." << std::endl;
+  std::cout << "     --ct3            print CT Private Data 2 (7005,10,TOSHIBA_MEC_CT3)." << std::endl;
+  std::cout << "     --pmtf           print PMTF INFORMATION DATA sub-sequences (0029,01,PMTF INFORMATION DATA)." << std::endl;
+  std::cout << "                      or TOSHIBA_MEC_MR3 sub-sequences (0029,01,TOSHIBA_MEC_MR3)." << std::endl;
+  std::cout << "                      or CANON_MEC_MR3 sub-sequences (0029,01,CANON_MEC_MR3)." << std::endl;
+  std::cout << "     --medcom         print MedCom History Information as UTF-8 (0029,20,SIEMENS MEDCOM HEADER)." << std::endl;
+  std::cout << "     --mr3            print Original Data as UTF-8 (700d,08,TOSHIBA_MEC_MR3)." << std::endl;
   std::cout << "  -A --asn1           print encapsulated ASN1 structure >(0400,0520)." << std::endl;
   std::cout << "     --map-uid-names  map UID to names." << std::endl;
   std::cout << "General Options:" << std::endl;
@@ -819,7 +1322,6 @@ void PrintHelp()
   std::cout << "  -I --ignore-errors   print even if file is corrupted." << std::endl;
 }
 
-
 int main (int argc, char *argv[])
 {
   int c;
@@ -830,10 +1332,19 @@ int main (int argc, char *argv[])
   int dump = 0;
   int print = 0;
   int printcsa = 0;
+  int printmrprotocol = 0;
+  int printcsabase64 = 0;
+  int printmedcom = 0; // MedCom History Information
+  int printcsaasl = 0;
+  int printcsadiffusion = 0;
+  int printcsa64 = 0;
   int printpdb = 0;
   int printelscint = 0;
   int printvepro = 0;
   int printsds = 0; // MR Series Data Storage
+  int printct3 = 0; // TOSHIBA_MEC_CT3
+  int printmecmr3 = 0; // TOSHIBA_MEC_MR3
+  int printpmtf = 0; // TOSHIBA / PMTF INFORMATION DATA & TOSHIBA / TOSHIBA_MEC_MR3 & CANON_MEC_MR3
   int verbose = 0;
   int warning = 0;
   int debug = 0;
@@ -843,7 +1354,7 @@ int main (int argc, char *argv[])
   int recursive = 0;
   int printasn1 = 0;
   int mapuidnames = 0;
-  while (1) {
+  while (true) {
     //int this_option_optind = optind ? optind : 1;
     int option_index = 0;
 /*
@@ -855,7 +1366,7 @@ int main (int argc, char *argv[])
           };
 */
     static struct option long_options[] = {
-        {"input", 1, 0, 0},
+        {"input", 1, nullptr, 0},
         {"xml-dict", 0, &printdict, 1},
         {"recursive", 0, &recursive, 1},
         {"print", 0, &print, 1},
@@ -875,7 +1386,16 @@ int main (int argc, char *argv[])
         {"elscint", 0, &printelscint, 1},
         {"vepro", 0, &printvepro, 1},
         {"sds", 0, &printsds, 1},
-        {0, 0, 0, 0} // required
+        {"ct3", 0, &printct3, 1},
+        {"csa-asl", 0, &printcsaasl, 1},
+        {"csa-diffusion", 0, &printcsadiffusion, 1},
+        {"csa64", 0, &printcsa64, 1},
+        {"mrprotocol", 0, &printmrprotocol, 1},
+        {"pmtf", 0, &printpmtf, 1},
+        {"mecmr3", 0, &printpmtf, 1},
+        {"medcom", 0, &printmedcom, 1},
+        {"mr3", 0, &printmecmr3, 1},
+        {nullptr, 0, nullptr, 0} // required
     };
     static const char short_options[] = "i:xrpdcCPAVWDEhvI";
     c = getopt_long (argc, argv, short_options,
@@ -890,7 +1410,7 @@ int main (int argc, char *argv[])
     case 0:
     case '-':
         {
-        const char *s = long_options[option_index].name;
+        const char *s = long_options[option_index].name; (void)s;
         //printf ("option %s", s);
         if (optarg)
           {
@@ -1046,6 +1566,28 @@ int main (int argc, char *argv[])
     std::cerr << "Not handled for now" << std::endl;
     }
 
+  std::vector<std::string> csanames;
+  if( printcsaasl )
+  {
+    printcsabase64 = 1;
+    csanames.push_back( "MR_ASL" );
+  }
+  else if( printcsadiffusion )
+  {
+    printcsabase64 = 1;
+    csanames.push_back( "MRDiffusion" );
+  }
+  else if( printcsa64 )
+  {
+    printcsabase64 = 1;
+    // keep me sorted
+    csanames.push_back( "FmriAcquisitionDescriptionSequence" );
+    csanames.push_back( "FmriConditionsDataSequence" );
+    csanames.push_back( "FmriResultSequence" );
+    csanames.push_back( "MR_ASL" );
+    csanames.push_back( "MRDiffusion" );
+  }
+
   // else
   int res = 0;
   if( !gdcm::System::FileExists(filename.c_str()) )
@@ -1076,6 +1618,22 @@ int main (int argc, char *argv[])
         {
         res += PrintSDS(*it, verbose!= 0);
         }
+      else if( printct3 )
+        {
+        res += PrintCT3(*it, verbose!= 0);
+        }
+      else if( printpmtf )
+        {
+        res += PrintPMTF(*it, verbose!= 0);
+        }
+      else if( printmecmr3 )
+        {
+        res += PrintMEC_MR3(*it, verbose!=0);
+        }
+      else if( printmedcom )
+        {
+        res += PrintMedComHistory(*it, verbose!=0);
+        }
       else if( printelscint )
         {
         res += PrintELSCINT(*it, verbose!= 0);
@@ -1088,6 +1646,14 @@ int main (int argc, char *argv[])
         {
         res += PrintCSA(*it);
         }
+      else if( printmrprotocol )
+        {
+        res += PrintMrProtocol(*it);
+        }
+      else if( printcsabase64 )
+        {
+        res += PrintCSABase64(*it, csanames);
+        }
       else if( dump )
         {
         res += DoOperation<gdcm::Dumper>(*it);
@@ -1119,6 +1685,22 @@ int main (int argc, char *argv[])
       {
       res += PrintSDS(filename, verbose!= 0);
       }
+    else if( printct3 )
+      {
+      res += PrintCT3(filename, verbose!= 0);
+      }
+    else if( printpmtf )
+      {
+      res += PrintPMTF(filename, verbose!= 0);
+      }
+    else if( printmecmr3 )
+      {
+      res += PrintMEC_MR3(filename, verbose!= 0);
+      }
+    else if( printmedcom )
+      {
+      res += PrintMedComHistory(filename, verbose!= 0);
+      }
     else if( printelscint )
       {
       res += PrintELSCINT(filename, verbose!= 0);
@@ -1131,6 +1713,14 @@ int main (int argc, char *argv[])
       {
       res += PrintCSA(filename);
       }
+    else if( printmrprotocol )
+      {
+      res += PrintMrProtocol(filename);
+      }
+    else if( printcsabase64 )
+      {
+      res += PrintCSABase64(filename, csanames);
+      }
     else if( dump )
       {
       res += DoOperation<gdcm::Dumper>(filename);
