@@ -30,6 +30,9 @@
 #include "gdcmMD5.h"
 #include "gdcmSystem.h"
 #include "gdcmDirectory.h"
+#include "gdcmImageHelper.h"
+#include "gdcmSplitMosaicFilter.h"
+#include "gdcmImageChangePlanarConfiguration.h"
 
 #ifdef GDCM_USE_SYSTEM_POPPLER
 #include <poppler/poppler-config.h>
@@ -52,7 +55,7 @@
 #include <sys/stat.h>
 
 
-int checkmagick(unsigned char *input)
+static int checkmagick(unsigned char *input)
 {
   if( input[128+0] == 'D'
    && input[128+1] == 'I'
@@ -64,7 +67,7 @@ int checkmagick(unsigned char *input)
   return 0;
 }
 
-int checkdeflated(const char *name)
+static int checkdeflated(const char *name)
 {
   int ret;
   unsigned char *source;
@@ -102,22 +105,22 @@ int checkdeflated(const char *name)
     return 1;
     }
   in = fopen(name, "r");
-  if (in == NULL)
+  if (in == nullptr)
     {
     fprintf( stderr, "in is NULL\n" );
     return 1;
     }
   buf = (unsigned char*)malloc(size);
-  if (buf != NULL && (size1 = fread(buf, 1, size, in)) != size) {
+  if (buf != nullptr && (size1 = (unsigned long)fread(buf, 1, size, in)) != size) {
     free(buf);
-    buf = NULL;
+    buf = nullptr;
     fprintf( stderr, "could not fread: %lu bytes != %lu\n", size, size1 );
     fprintf( stderr, "feof: %i ferror %i\n", feof(in), ferror(in) );
   }
   fclose(in);
   len = size;
   source = buf;
-  if( source == NULL ) {
+  if( source == nullptr ) {
     fprintf( stderr, "source is NULL\n" );
     return 1;
   }
@@ -169,7 +172,7 @@ int checkdeflated(const char *name)
     printf( "deflate stream has proper length: %lu\n", len );
     }
 
-  ret = puff(NULL, &destlen, source, &sourcelen);
+  ret = puff(nullptr, &destlen, source, &sourcelen);
 
   if (ret)
     fprintf(stdout,"puff() failed with return code %d\n", ret);
@@ -183,18 +186,30 @@ int checkdeflated(const char *name)
 }
 
 #ifdef GDCM_USE_SYSTEM_POPPLER
-std::string getInfoDate(Dict *infoDict, const char *key)
+static std::string getInfoDate(Dict *infoDict, const char *key)
 {
   Object obj;
-  char *s;
+  const char *s;
   int year, mon, day, hour, min, sec, n;
   struct tm tmStruct;
-  char buf[256];
+  //char buf[256];
   std::string out;
 
+#ifdef LIBPOPPLER_NEW_OBJECT_API
+  if ((obj = infoDict->lookup(const_cast<char*>(key))).isString())
+#else
   if (infoDict->lookup((char*)key, &obj)->isString())
+#endif
     {
-    s = obj.getString()->getCString();
+#ifndef LIBPOPPLER_GOOSTRING_HAS_GETCSTRING
+    const
+#endif
+    GooString* gs = obj.getString();
+#ifdef LIBPOPPLER_GOOSTRING_HAS_GETCSTRING
+    s = gs->getCString();
+#else
+    s = gs->c_str();
+#endif
     if (s[0] == 'D' && s[1] == ':')
       {
       s += 2;
@@ -204,11 +219,11 @@ std::string getInfoDate(Dict *infoDict, const char *key)
       {
       switch (n)
         {
-      case 1: mon = 1;
-      case 2: day = 1;
-      case 3: hour = 0;
-      case 4: min = 0;
-      case 5: sec = 0;
+      case 1: mon = 1;  /* fall through */
+      case 2: day = 1;  /* fall through */
+      case 3: hour = 0; /* fall through */
+      case 4: min = 0;  /* fall through */
+      case 5: sec = 0;  /* fall through */
         }
       tmStruct.tm_year = year - 1900;
       tmStruct.tm_mon = mon - 1;
@@ -240,32 +255,46 @@ std::string getInfoDate(Dict *infoDict, const char *key)
         out = date;
       }
     }
+#ifndef LIBPOPPLER_NEW_OBJECT_API
   obj.free();
+#endif
   return out;
 }
 
-std::string getInfoString(Dict *infoDict, const char *key, UnicodeMap *uMap)
+#ifdef LIBPOPPLER_UNICODEMAP_HAS_CONSTMAPUNICODE
+static std::string getInfoString(Dict *infoDict, const char *key, const UnicodeMap *uMap)
+#else
+static std::string getInfoString(Dict *infoDict, const char *key, UnicodeMap *uMap)
+#endif
 {
   Object obj;
+#ifdef LIBPOPPLER_GOOSTRING_HAS_CONSTGETCHAR
+  const GooString *s1;
+#else
   GooString *s1;
-  GBool isUnicode;
+#endif
+  bool isUnicode;
   Unicode u;
   char buf[8];
   int i, n;
   std::string out;
 
+#ifdef LIBPOPPLER_NEW_OBJECT_API
+  if ((obj = infoDict->lookup(const_cast<char*>(key))).isString())
+#else
   if (infoDict->lookup((char*)key, &obj)->isString())
+#endif
     {
     s1 = obj.getString();
     if ((s1->getChar(0) & 0xff) == 0xfe &&
       (s1->getChar(1) & 0xff) == 0xff)
       {
-      isUnicode = gTrue;
+      isUnicode = true;
       i = 2;
       }
     else
       {
-      isUnicode = gFalse;
+      isUnicode = false;
       i = 0;
       }
     while (i < obj.getString()->getLength())
@@ -286,20 +315,22 @@ std::string getInfoString(Dict *infoDict, const char *key, UnicodeMap *uMap)
       out.append( std::string(buf, n) );
       }
     }
+#ifndef LIBPOPPLER_NEW_OBJECT_API
   obj.free();
+#endif
   return out;
 }
 #endif
 
 
-void PrintVersion()
+static void PrintVersion()
 {
   std::cout << "gdcminfo: gdcm " << gdcm::Version::GetVersion() << " ";
   const char date[] = "$Date$";
   std::cout << date << std::endl;
 }
 
-void PrintHelp()
+static void PrintHelp()
 {
   PrintVersion();
   std::cout << "Usage: gdcminfo [OPTION]... FILE..." << std::endl;
@@ -316,6 +347,10 @@ void PrintHelp()
   //std::cout << "     --media-storage-uid   return media storage uid only." << std::endl;
   //std::cout << "     --media-storage-name  return media storage name only (when possible)." << std::endl;
 //  std::cout << "  -b --check-big-endian   check if file is ." << std::endl;
+  std::cout << "     --force-rescale    force rescale." << std::endl;
+  std::cout << "     --force-spacing    force spacing." << std::endl;
+  std::cout << "     --mosaic           dump image information of MOSAIC." << std::endl;
+
   std::cout << "General Options:" << std::endl;
   std::cout << "  -V --verbose   more verbose (warning+error)." << std::endl;
   std::cout << "  -W --warning   print warning info." << std::endl;
@@ -330,8 +365,9 @@ void PrintHelp()
   int deflated = 0; // check deflated
   int checkcompression = 0;
   int md5sum = 0;
+  int mosaic = 0;
 
-int ProcessOneFile( std::string const & filename, gdcm::Defs const & defs )
+static int ProcessOneFile( std::string const & filename, gdcm::Defs const & defs )
 {
   (void)defs;
   if( deflated )
@@ -378,34 +414,63 @@ int ProcessOneFile( std::string const & filename, gdcm::Defs const & defs )
       std::cerr << "Could not read image from: " << filename << std::endl;
       return 1;
       }
-    //const gdcm::File &file = reader.GetFile();
-    //const gdcm::DataSet &ds = file.GetDataSet();
+    gdcm::SplitMosaicFilter filter;
+    const gdcm::Image *pimage = nullptr;
     const gdcm::Image &image = reader.GetImage();
-    const double *dircos = image.GetDirectionCosines();
+    if( mosaic )
+    {
+      filter.SetImage( image );
+      filter.SetFile( reader.GetFile() );
+      if( !filter.Split() )
+      {
+      std::cerr << "Could not split mosaic : " << filename << std::endl;
+      return 1;
+      }
+      pimage = &filter.GetImage();
+    }
+    else
+    {
+    pimage = &image;
+    }
+    const double *dircos = pimage->GetDirectionCosines();
     gdcm::Orientation::OrientationType type = gdcm::Orientation::GetType(dircos);
     const char *label = gdcm::Orientation::GetLabel( type );
-    image.Print( std::cout );
+    pimage->Print( std::cout );
     std::cout << "Orientation Label: " << label << std::endl;
     if( checkcompression )
       {
-      bool lossy = image.IsLossy();
+      bool lossy = pimage->IsLossy();
       std::cout << "Encapsulated Stream was found to be: " << (lossy ? "lossy" : "lossless") << std::endl;
       }
 
     if( md5sum )
       {
-      char *buffer = new char[ image.GetBufferLength() ];
-      if( image.GetBuffer( buffer ) )
+      int ret = 0;
+      char *buffer = new char[ pimage->GetBufferLength() ];
+      gdcm::ImageChangePlanarConfiguration icpc;
+      icpc.SetPlanarConfiguration( 0 );
+      icpc.SetInput( *pimage );
+      bool b = icpc.Change();
+      if( !b )
+        {
+        std::cerr << "Could not change the Planar Configuration: " << filename << std::endl;
+        return 1;
+        }
+      pimage = &icpc.GetOutput();
+
+      if( pimage->GetBuffer( buffer ) )
         {
         char digest[33] = {};
-        gdcm::MD5::Compute( buffer, image.GetBufferLength(), digest );
+        gdcm::MD5::Compute( buffer, pimage->GetBufferLength(), digest );
         std::cout << "md5sum: " << digest << std::endl;
         }
       else
         {
-        std::cout << "Problem decompressing file: " << filename << std::endl;
+        std::cerr << "Problem decompressing file: " << filename << std::endl;
+        ret = 1;
         }
       delete[] buffer;
+      return ret;
       }
     }
   else if ( ms == gdcm::MediaStorage::EncapsulatedPDFStorage )
@@ -414,7 +479,7 @@ int ProcessOneFile( std::string const & filename, gdcm::Defs const & defs )
     const gdcm::DataSet &ds = file.GetDataSet();
     const gdcm::DataElement& de = ds.GetDataElement( gdcm::Tag(0x42,0x11) );
     const gdcm::ByteValue* bv = de.GetByteValue();
-    const char *p = bv->GetPointer();
+    const char *p = bv->GetPointer(); (void)p;
     Object appearDict;
     //appearDict.initDict(xref);
     //appearDict.dictAdd(copyString("Length"),
@@ -429,11 +494,19 @@ int ProcessOneFile( std::string const & filename, gdcm::Defs const & defs )
 
     MemStream *appearStream;
 
-    appearStream = new MemStream((char*)bv->GetPointer(), 0,
+    appearStream = new MemStream(const_cast<char*>(bv->GetPointer()), 0,
+#ifdef LIBPOPPLER_NEW_OBJECT_API
+      bv->GetLength(), std::move(appearDict));
+#else
       bv->GetLength(), &appearDict);
+#endif
+#ifdef LIBPOPPLER_PDFDOC_HAS_OPTIONAL
+    std::optional<GooString> ownerPW, userPW;
+#else
     GooString *ownerPW, *userPW;
     ownerPW = NULL;
     userPW = NULL;
+#endif
 
     PDFDoc *doc;
     doc = new PDFDoc(appearStream, ownerPW, userPW);
@@ -447,18 +520,30 @@ int ProcessOneFile( std::string const & filename, gdcm::Defs const & defs )
     std::string creationdate;
     std::string moddate;
 
+#ifdef LIBPOPPLER_UNICODEMAP_HAS_CONSTMAPUNICODE
+    const UnicodeMap *uMap;
+#else
     UnicodeMap *uMap;
+#endif
 #ifdef LIBPOPPLER_GLOBALPARAMS_CSTOR_HAS_PARAM
     globalParams = new GlobalParams(0);
+#else
+#ifdef LIBPOPPLER_GLOBALPARAMS_HAS_RESET
+    globalParams.reset(new GlobalParams());
 #else
     globalParams = new GlobalParams();
+#endif
 #endif
     uMap = globalParams->getTextEncoding();
 
     Object info;
     if (doc->isOk())
       {
+#ifdef LIBPOPPLER_NEW_OBJECT_API
+      info = doc->getDocInfo();
+#else
       doc->getDocInfo(&info);
+#endif
       if (info.isDict())
         {
         title        = getInfoString(info.getDict(), "Title",    uMap);
@@ -469,16 +554,22 @@ int ProcessOneFile( std::string const & filename, gdcm::Defs const & defs )
         producer     = getInfoString(info.getDict(), "Producer", uMap);
         creationdate = getInfoDate(  info.getDict(), "CreationDate"  );
         moddate      = getInfoDate(  info.getDict(), "ModDate"       );
+#ifndef LIBPOPPLER_NEW_OBJECT_API
         info.free();
+#endif
         }
+#ifdef LIBPOPPLER_CATALOG_HAS_STRUCTTREEROOT
+      const char *tagged = doc->getStructTreeRoot() ? "yes" : "no";
+#else
       const char *tagged = doc->getStructTreeRoot()->isDict() ? "yes" : "no";
+#endif
       int pages = doc->getNumPages();
       const char *encrypted = doc->isEncrypted() ? "yes" : "no";
       //  printf("yes (print:%s copy:%s change:%s addNotes:%s)\n",
-      //   doc->okToPrint(gTrue) ? "yes" : "no",
-      //   doc->okToCopy(gTrue) ? "yes" : "no",
-      //   doc->okToChange(gTrue) ? "yes" : "no",
-      //   doc->okToAddNotes(gTrue) ? "yes" : "no");
+      //   doc->okToPrint(true) ? "yes" : "no",
+      //   doc->okToCopy(true) ? "yes" : "no",
+      //   doc->okToChange(true) ? "yes" : "no",
+      //   doc->okToAddNotes(true) ? "yes" : "no");
 
       // print linearization info
       const char *optimized = doc->isLinearized() ? "yes" : "no";
@@ -487,7 +578,7 @@ int ProcessOneFile( std::string const & filename, gdcm::Defs const & defs )
 #ifdef LIBPOPPLER_PDFDOC_HAS_PDFVERSION
       float pdfversion = doc->getPDFVersion();
 #else
-      float pdfversion = doc->getPDFMajorVersion() + 0.1 * doc->getPDFMinorVersion();
+      const double pdfversion = doc->getPDFMajorVersion() + 0.1 * doc->getPDFMinorVersion();
 #endif
 
 
@@ -533,6 +624,9 @@ int main(int argc, char *argv[])
   int c;
   std::string filename;
   std::string xmlpath;
+  int forcerescale = 0;
+  int forcespacing = 0;
+
   int resourcespath = 0;
   int verbose = 0;
   int warning = 0;
@@ -541,15 +635,18 @@ int main(int argc, char *argv[])
   int version = 0;
   int debug = 0;
   int error = 0;
-  while (1) {
+  while (true) {
     int option_index = 0;
     static struct option long_options[] = {
-        {"input", 1, 0, 0},
+        {"input", 1, nullptr, 0},
         {"recursive", 0, &recursive, 1},
         {"check-deflated", 0, &deflated, 1},
         {"resources-path", 0, &resourcespath, 1},
         {"md5sum", 0, &md5sum, 1},
         {"check-compression", 0, &checkcompression, 1},
+        {"force-rescale", 0, &forcerescale, 1},
+        {"force-spacing", 0, &forcespacing, 1},
+        {"mosaic", 0, &mosaic, 1},
 
         {"verbose", 0, &verbose, 1},
         {"warning", 0, &warning, 1},
@@ -557,7 +654,7 @@ int main(int argc, char *argv[])
         {"error", 0, &error, 1},
         {"help", 0, &help, 1},
         {"version", 0, &version, 1},
-        {0, 0, 0, 0} // required
+        {nullptr, 0, nullptr, 0} // required
     };
     static const char short_options[] = "i:rdVWDEhv";
     c = getopt_long (argc, argv, short_options,
@@ -572,7 +669,7 @@ int main(int argc, char *argv[])
     case 0:
     case '-':
         {
-        const char *s = long_options[option_index].name;
+        const char *s = long_options[option_index].name; (void)s;
         //printf ("option %s", s);
         if (optarg)
           {
@@ -670,6 +767,9 @@ int main(int argc, char *argv[])
     return 0;
     }
 
+  gdcm::ImageHelper::SetForceRescaleInterceptSlope(forcerescale ? true : false);
+  gdcm::ImageHelper::SetForcePixelSpacing(forcespacing ? true : false);
+
   if( filename.empty() )
     {
     //std::cerr << "Need input file (-i)\n";
