@@ -42,7 +42,7 @@ int main(int , char *[])
 
   // 3. This solution is the most flexible solution as you can request using the same
   // API either a public tag or a private tag
-  const char *strowner = 0;
+  const char *strowner = nullptr;
   const gdcm::DictEntry &entry3 = dicts.GetDictEntry(patient_name,strowner);
   std::cout << entry3 << std::endl;
 
@@ -56,7 +56,7 @@ int main(int , char *[])
 
   // Let's pretend that private lookup is on 0x10xx elements:
   gdcm::PrivateTag dummy = private_tag;
-  dummy.SetElement( 0x1000 + dummy.GetElement() );
+  dummy.SetElement( (uint16_t)(0x1000 + dummy.GetElement()) );
   const gdcm::DictEntry &entry5 = dicts.GetDictEntry(dummy,dummy.GetOwner());
   std::cout << entry5 << std::endl;
 
