@@ -239,6 +239,15 @@ public class ScanDirectory
       {
       throw new Exception("Could not scan");
       }
+    String fn0 = fns.get(0);
+    TagToValue mappings = s.GetMapping( fn0 );
+    System.out.println( "mappings size: " + mappings.size() );
+    for( Tag tag : tagarray ) {
+      if( mappings.has_key( tag ) ) {
+        String val = mappings.get( tag );
+        System.out.println( "tag/val: " + tag + "->" + val );
+      }
+    }
 
     for( long idx = 0; idx < fns.size(); ++idx )
       {
@@ -249,6 +258,7 @@ public class ScanDirectory
       TagSetType tst = new TagSetType();
       tst.insert( new Tag(0x7fe0,0x10) );
       b = r.ReadUpToTag( new Tag(0x88,0x200), tst );
+      UIntArrayType dims = ImageHelper.GetDimensionsValue( r.GetFile() );
       if( b )
         {
         IconImageFilter iif = new IconImageFilter();
