@@ -1,9 +1,8 @@
 /*=========================================================================
 
   Program: GDCM (Grassroots DICOM). A DICOM library
-  Module:  $URL$
 
-  Copyright (c) 2006-2010 Mathieu Malaterre
+  Copyright (c) 2006-2011 Mathieu Malaterre
   All rights reserved.
   See Copyright.txt or http://gdcm.sourceforge.net/Copyright.html for details.
 
@@ -92,7 +91,7 @@ void DictConverter::Convert()
   // Keep binary mode !!!
   from.open( Internal->InputFilename.c_str(), std::ios::binary);
   std::ofstream &into = Internal->OutputStream; //(outfilename);
-  into.open( Internal->OutputFilename.c_str() );
+  into.open( Internal->OutputFilename.c_str(), std::ios::binary );
 
   WriteHeader();
 
@@ -227,7 +226,7 @@ bool DictConverter::Readuint16(const char *raw, uint16_t &ov)
   int r = sscanf(raw, "%04x", &v);
   assert( r == 1 && "Wrong Value read for uint16");
   char sv[4+1];
-  r = sprintf(sv, "%04x", v);
+  r = snprintf(sv, sizeof(sv), "%04x", v);
   assert( r == 4 && "Wrong Value printed for uint16");
   assert( strncmp(raw, sv, 4) == 0 );
   ov = v;
@@ -310,10 +309,10 @@ bool DictConverter::ConvertToXML(const char *raw, std::string &cxx)
   ReadVM(raw+len, vm);
   len += strlen(VM::GetVMString(vm))+1;
   std::ostringstream os;
-  os << "  <DataElement Tag=\"" << std::hex << std::setw(4) 
-    << std::setfill('0') << group << "," 
+  os << "  <DataElement Tag=\"" << std::hex << std::setw(4)
+    << std::setfill('0') << group << ","
     << std::setw(4) << std::setfill('0')
-    << element << "\" VR=\"" << vr << "\" VM=\"" << vm << "\"\tName=\"" 
+    << element << "\" VR=\"" << vr << "\" VM=\"" << vm << "\"\tName=\""
     << (raw+len) << "\">";
   cxx = os.str();
   return true;
@@ -340,17 +339,17 @@ bool DictConverter::ConvertToCXX(const char *raw, std::string &cxx)
   ReadVM(raw+len, vm);
   len += strlen(VM::GetVMString(vm))+1;
   std::ostringstream os;
-  os << "   {0x" << std::hex << std::setw(4) << std::setfill('0') 
-    << group << ", 0x" 
+  os << "   {0x" << std::hex << std::setw(4) << std::setfill('0')
+    << group << ", 0x"
     << std::setw(4) << std::setfill('0');
   if( OutputType == DICT_DEBUG )
     {
-    os << element << ", \"" << vr << "\" , \"" << vm << "\" , \"" 
+    os << element << ", \"" << vr << "\" , \"" << vm << "\" , \""
       << (raw+len) << "\"}, ";
     }
   else
     {
-    os << element << std::dec << ",(VR::VRType)" << (int)vr 
+    os << element << std::dec << ",(VR::VRType)" << (int)vr
       << ",(VM::VMType)" << (int)vm << ",\"" << (raw+len) << "\"},";
     }
   cxx = os.str();
@@ -358,7 +357,7 @@ bool DictConverter::ConvertToCXX(const char *raw, std::string &cxx)
 }
 
 
-void DictConverter::AddGroupLength() 
+void DictConverter::AddGroupLength()
 {
   std::ifstream &from = Internal->InputStream;
   std::ofstream &into = Internal->OutputStream;
