@@ -60,7 +60,7 @@ namespace gdcm
 {"1.2.840.10008.5.1.4.1.1.12.2.1" , "Enhanced X Ray RF Image IOD Modules"},
 {"1.2.840.10008.5.1.4.1.1.13.1.1" , "X Ray 3D Angiographic Image IOD Modules"},
 {"1.2.840.10008.5.1.4.1.1.13.1.2" , "X-Ray 3D Craniofacial Image"},
-{"1.2.840.10008.5.1.4.1.1.13.1.3" , "IOD defined in PS 3.3"},
+{"1.2.840.10008.5.1.4.1.1.13.1.3" , "Breast Tomosynthesis Image IOD Modules"},
 {"1.2.840.10008.5.1.4.1.1.20" , "NM Image IOD Modules"},
 {"1.2.840.10008.5.1.4.1.1.66" , "Raw Data IOD Modules"},
 {"1.2.840.10008.5.1.4.1.1.66.1" , "Spatial Registration IOD Modules"},
@@ -69,9 +69,9 @@ namespace gdcm
 {"1.2.840.10008.5.1.4.1.1.66.4" , "Segmentation IOD Modules"},
 {"1.2.840.10008.5.1.4.1.1.66.5" , "Surface Segmentation"},
 {"1.2.840.10008.5.1.4.1.1.67" , "Real World Value Mapping"},
-{"1.2.840.10008.5.1.4.1.1.77.1.1" , "VL Endoscopic Image"},
+{"1.2.840.10008.5.1.4.1.1.77.1.1" , "VL Endoscopic Image IOD Modules"},
 {"1.2.840.10008.5.1.4.1.1.77.1.1.1" , "Video Endoscopic Image IOD Modules"},
-{"1.2.840.10008.5.1.4.1.1.77.1.2" , "VL Microscopic Image"},
+{"1.2.840.10008.5.1.4.1.1.77.1.2" , "VL Microscopic Image IOD Modules"},
 {"1.2.840.10008.5.1.4.1.1.77.1.2.1" , "Video Microscopic Image"},
 {"1.2.840.10008.5.1.4.1.1.77.1.3" , "VL Slide-Coordinates Microscopic Image"},
 {"1.2.840.10008.5.1.4.1.1.77.1.4" , "VL Photographic Image IOD Modules"},
@@ -94,12 +94,12 @@ namespace gdcm
 {"1.2.840.10008.5.1.4.1.1.88.50" , "Mammography CAD SR IOD Modules"},
 {"1.2.840.10008.5.1.4.1.1.88.59" , "Key Object Selection Document IOD Modules"},
 {"1.2.840.10008.5.1.4.1.1.88.65" , "Chest CAD SR IOD"},
-{"1.2.840.10008.5.1.4.1.1.88.67" , "X-Ray Radiation Dose SR"},
+{"1.2.840.10008.5.1.4.1.1.88.67" , "X Ray Radiation Dose SR IOD Modules"},
 {"1.2.840.10008.5.1.4.1.1.88.69" , "Colon CAD SR IOD"},
 {"1.2.840.10008.5.1.4.1.1.104.1" , "Encapsulated PDF IOD Modules"},
-{"1.2.840.10008.5.1.4.1.1.104.2" , "Encapsulated CDA IOD"},
+{"1.2.840.10008.5.1.4.1.1.104.2" , "Encapsulated CDA IOD Modules"},
 {"1.2.840.10008.5.1.4.1.1.128" , "PET Image IOD Modules"},
-{"1.2.840.10008.5.1.4.1.1.130" , "IOD defined in PS 3.3"},
+{"1.2.840.10008.5.1.4.1.1.130" , "Enhanced PET Image Storage"},
 {"1.2.840.10008.5.1.4.1.1.131" , "Basic Structured Display IOD"},
 {"1.2.840.10008.5.1.4.1.1.481.1" , "RT Image IOD Modules"},
 {"1.2.840.10008.5.1.4.1.1.481.2" , "RT Dose IOD Modules"},
@@ -112,14 +112,24 @@ namespace gdcm
 {"1.2.840.10008.5.1.4.1.1.481.9" , "RT Ion Beams Treatment Record IOD Modules"},
 {"1.2.840.10008.5.1.4.38.1" , "Hanging Protocol IOD Modules"},
 {"1.2.840.10008.5.1.4.39.1" , "Color Palette IOD"},
+{"1.2.840.10008.5.1.4.1.1.2.2", "Legacy Converted Enhanced CT Image IOD Modules"},
+{"1.2.840.10008.5.1.4.1.1.4.4", "Legacy Converted Enhanced MR Image IOD Modules"},
+{"1.2.840.10008.5.1.4.1.1.128.1", "Legacy Converted Enhanced PET Image IOD Modules"},
+{"1.2.840.10008.5.1.4.1.1.13.1.4" , "Breast Projection X-Ray Image - For Presentation Image IOD Modules"},
+{"1.2.840.10008.5.1.4.1.1.13.1.5" , "Breast Projection X-Ray Image - For Processing Image IOD Modules"},
+
 // Deprecated:
 {"1.2.840.10008.3.1.2.3.3" , "Modality Performed Procedure Step IOD Modules" },
 {"1.2.840.10008.5.1.4.1.1.5" , "NM Image IOD Modules"},
 {"1.2.840.10008.5.1.4.1.1.6" , "US Image IOD Modules"},
 {"1.2.840.10008.5.1.4.1.1.3" , "US Multi Frame Image IOD Modules"},
-{"1.2.840.10008.5.1.4.1.1.12.3" , ""},
+{"1.2.840.10008.5.1.4.1.1.12.3" , ""}, // XRayAngiographicBiplaneImageStorage
+// private:
+{ "1.3.12.2.1107.5.9.1" , "Siemens Non-image IOD Modules"}, // CSA Non-Image Storage
+{ "1.2.392.200036.9125.1.1.2" , "Fuji Private CR Image IOD Modules"}, // 
+{ "1.2.392.200036.9125.1.1.4" , "Fuji Private Mammo CR Image IOD Modules"}, // 
 
-{ 0, 0 }
+{ nullptr, nullptr }
 };
 
 unsigned int SOPClassUIDToIOD::GetNumberOfSOPClassToIOD()
@@ -178,18 +188,18 @@ SOPClassUIDToIOD::SOPClassUIDToIODType& SOPClassUIDToIOD::GetSOPClassUIDToIOD(un
   if( i < SOPClassUIDToIOD::GetNumberOfSOPClassToIOD() )
     return SOPClassUIDToIODStrings[i];
   // else return the {0x0, 0x0} sentinel:
-  assert( *SOPClassUIDToIODStrings[ SOPClassUIDToIOD::GetNumberOfSOPClassToIOD() ] == 0 );
+  assert( *SOPClassUIDToIODStrings[ SOPClassUIDToIOD::GetNumberOfSOPClassToIOD() ] == nullptr );
   return SOPClassUIDToIODStrings[ SOPClassUIDToIOD::GetNumberOfSOPClassToIOD() ];
 
 }
 
 const char *SOPClassUIDToIOD::GetSOPClassUIDFromIOD(const char *iod)
 {
-  if(!iod) return NULL;
+  if(!iod) return nullptr;
   unsigned int i = 0;
   SOPClassUIDToIODType *sopclassuidtoiods = GetSOPClassUIDToIODs();
   const char *p = sopclassuidtoiods[i][1];
-  while( p != 0 )
+  while( p != nullptr )
     {
     if( strcmp( iod, p ) == 0 )
       {
@@ -205,11 +215,11 @@ const char *SOPClassUIDToIOD::GetSOPClassUIDFromIOD(const char *iod)
 
 const char *SOPClassUIDToIOD::GetIODFromSOPClassUID(const char *sopclassuid)
 {
-  if(!sopclassuid) return NULL;
+  if(!sopclassuid) return nullptr;
   unsigned int i = 0;
   SOPClassUIDToIODType *sopclassuidtoiods = GetSOPClassUIDToIODs();
   const char *p = sopclassuidtoiods[i][0];
-  while( p != 0 )
+  while( p != nullptr )
     {
     if( strcmp( sopclassuid, p ) == 0 )
       {
