@@ -20,7 +20,7 @@
 #include "gdcmImplicitDataElement.h"
 #include "gdcmTrace.h"
 
-namespace gdcm
+namespace gdcm_ns
 {
   void DataElement::SetVLToUndefined() {
     assert( VRField == VR::SQ || VRField == VR::INVALID
@@ -34,34 +34,14 @@ namespace gdcm
     ValueLengthField.SetToUndefined();
   }
 
-#if !defined(GDCM_LEGACY_REMOVE)
-  SequenceOfItems* DataElement::GetSequenceOfItems() {
-    GDCM_LEGACY_REPLACED_BODY(DataElement::GetSequenceOfItems, "GDCM 2.2",
-                              DataElement::GetValueAsSQ);
-    SequenceOfItems *sqi = dynamic_cast<SequenceOfItems*>(ValueField.GetPointer());
-    if(!sqi)
-      {
-      // Was the element loaded as a byte value ? Let's check:
-      assert( IsEmpty() );
-      }
-    return sqi;
-  }
-  const SequenceOfItems* DataElement::GetSequenceOfItems() const {
-    GDCM_LEGACY_REPLACED_BODY(DataElement::GetSequenceOfItems, "GDCM 2.2",
-                              DataElement::GetValueAsSQ);
-    const SequenceOfItems *sqi = dynamic_cast<SequenceOfItems*>(ValueField.GetPointer());
-    if(!sqi)
-      {
-      // Was the element loaded as a byte value ? Let's check:
-      assert( IsEmpty() );
-      }
-    return sqi;
-  }
-#endif
   const SequenceOfFragments* DataElement::GetSequenceOfFragments() const {
     const SequenceOfFragments *sqf = dynamic_cast<SequenceOfFragments*>(ValueField.GetPointer());
     return sqf;
   }
+  SequenceOfFragments* DataElement::GetSequenceOfFragments() {
+    SequenceOfFragments *sqf = dynamic_cast<SequenceOfFragments*>(ValueField.GetPointer());
+    return sqf;
+  }
 
   /*
    *  Two cases are handled:
@@ -76,7 +56,7 @@ namespace gdcm
     {
     if( IsEmpty() /*|| GetByteValue()*/ || GetSequenceOfFragments() )
       {
-      return 0;
+      return nullptr;
       }
     SequenceOfItems *sq = dynamic_cast<SequenceOfItems*>(ValueField.GetPointer());
     if( sq ) // all set !
@@ -100,7 +80,7 @@ namespace gdcm
             {
             std::stringstream ss;
             ss.str( s );
-            sqi->Read<ImplicitDataElement,SwapperNoOp>( ss );
+            sqi->Read<ImplicitDataElement,SwapperNoOp>( ss, true );
             }
           catch(Exception &ex)
             {
@@ -114,11 +94,18 @@ namespace gdcm
             ss.str(s);
             Tag item;
             item.Read<SwapperNoOp>(ss);
-            assert( item == itemPMSStart );
-            ss.seekg(-4,std::ios::cur);
-            sqi->Read<ExplicitDataElement,SwapperDoOp>( ss );
-			(void)ex;  //to avoid unreferenced variable warning on release
-			gdcmErrorMacro(ex.what());
+            if( item == itemPMSStart )
+              {
+              ss.seekg(-4,std::ios::cur);
+              sqi->Read<ExplicitDataElement,SwapperDoOp>( ss, true );
+              gdcmWarningMacro(ex.what());
+              (void)ex;  //to avoid unreferenced variable warning on release
+              }
+            else
+              {
+              delete sqi;
+              sqi = nullptr;
+              }
             }
           return sqi;
           }
@@ -134,27 +121,69 @@ namespace gdcm
             {
             std::stringstream ss;
             ss.str( s );
-            sqi->Read<ImplicitDataElement,SwapperNoOp>( ss );
+            sqi->Read<ImplicitDataElement,SwapperNoOp>( ss, true );
             }
-          catch ( Exception & )
+          catch ( Exception &ex0 )
             {
             // Some people like to skew things up and write invalid SQ in VR:UN field
             // if conversion fails, simply keep the binary VR:UN stuff as-is
-            // eg. AMIInvalidPrivateDefinedLengthSQasUN.dcm";
+            // eg. AMIInvalidPrivateDefinedLengthSQasUN.dcm
             //const char *s = e.what();
             // s -> gdcm::ImplicitDataElement -> Impossible (more)
-            std::stringstream ss;
-            ss.str(s);
             try
               {
-              sqi->Read<ExplicitDataElement,SwapperDoOp>( ss );
+              std::stringstream ss;
+              ss.str(s);
+              sqi->Read<ExplicitDataElement,SwapperDoOp>( ss, true );
+              gdcmWarningMacro(ex0.what()); (void)ex0;
               }
-            catch ( Exception & )
+            catch ( Exception &ex1 )
               {
-              gdcmErrorMacro( "Could not read SQ. Giving up" );
-              throw "Could not decode this SQ";
-              return NULL;
+              // Let's try again this time but without the byte swapping option:
+              // eg. MEDILABInvalidCP246_EVRLESQasUN.dcm
+              try
+                {
+                std::stringstream ss;
+                ss.str(s);
+                sqi->Read<ExplicitDataElement,SwapperNoOp>( ss, true );
+                gdcmWarningMacro(ex1.what()); (void)ex1;
+                }
+              catch ( Exception &ex2 )
+                {
+                gdcmErrorMacro( "Could not read SQ. Giving up" );
+                gdcmErrorMacro(ex2.what()); (void)ex2;
+                delete sqi;
+                return nullptr;
+                }
               }
+            catch ( ... )
+              {
+              gdcmErrorMacro( "Could not read SQ, unknown exception" );
+              delete sqi;
+              return nullptr;
+              }
+            }
+          return sqi;
+          }
+        else if  ( GetVR() & VR::OB_OW ) // pre-dicom 1993 ?
+          {
+          const ByteValue *bv = GetByteValue();
+          assert( bv );
+          SequenceOfItems *sqi = new SequenceOfItems;
+          sqi->SetLength( bv->GetLength() );
+          std::string s( bv->GetPointer(), bv->GetLength() );
+          try
+            {
+            std::stringstream ss;
+            ss.str( s );
+            sqi->Read<ImplicitDataElement,SwapperNoOp>( ss, true );
+            }
+          catch ( Exception &ex0 )
+            {
+            gdcmErrorMacro( "Could not read SQ as OB. Giving up" );
+            gdcmErrorMacro(ex0.what()); (void)ex0;
+            delete sqi;
+            return nullptr;
             }
           return sqi;
           }
@@ -162,7 +191,7 @@ namespace gdcm
           {
           assert( GetVR().IsVRFile() );
           assert( GetByteValue() );
-          return 0;
+          return nullptr;
           }
         }
       }
@@ -178,8 +207,14 @@ namespace gdcm
     //      {
     //      gdcmWarningMacro( "Unknown exception" );
     //      }
-
-    return 0;
     }
 
-} // end namespace gdcm
+void DataElement::SetValueFieldLength( VL vl, bool readvalues )
+{
+  if( readvalues )
+    ValueField->SetLength(vl); // perform realloc
+  else
+    ValueField->SetLengthOnly(vl); // do not perform realloc
+}
+
+} // end namespace gdcm_ns
