@@ -29,7 +29,7 @@
 #include <cmath>
 #include <cstring>
 
-namespace gdcm
+namespace gdcm_ns
 {
 
 // Forward declaration
@@ -38,28 +38,52 @@ namespace gdcm
  *
  * \note TODO
  */
-template<int T> class EncodingImplementation;
+template<long long T> class EncodingImplementation;
+
+
+/**
+ *  \brief A class which is used to produce compile errors for an
+ * invalid combination of template parameters.
+ *
+ * Invalid combinations have specialized declarations with no
+ * definition.
+ */
+template <long long TVR, int TVM>
+class ElementDisableCombinations {};
+template <>
+class  ElementDisableCombinations<VR::OB, VM::VM1_n> {};
+template <>
+class  ElementDisableCombinations<VR::OW, VM::VM1_n> {};
+// Make it impossible to compile these other cases
+template <int TVM>
+class  ElementDisableCombinations<VR::OB, TVM>;
+template <int TVM>
+class ElementDisableCombinations<VR::OW, TVM>;
 
 /**
  * \brief Element class
  *
  * \note TODO
  */
-template<int TVR, int TVM>
+template<long long TVR, int TVM>
 class Element
 {
+  enum { ElementDisableCombinationsCheck = sizeof ( ElementDisableCombinations<TVR, TVM> ) };
 public:
   typename VRToType<TVR>::Type Internal[VMToLength<TVM>::Length];
   typedef typename VRToType<TVR>::Type Type;
 
+  static VR  GetVR()  { return (VR::VRType)TVR; }
+  static VM  GetVM()  { return (VM::VMType)TVM; }
+
   unsigned long GetLength() const {
     return VMToLength<TVM>::Length;
   }
   // Implementation of Print is common to all Mode (ASCII/Binary)
   // TODO: Can we print a \ when in ASCII...well I don't think so
-  // it would mean we used a bad VM then, right ?
+  // it would mean we used a bad VM then, right?
   void Print(std::ostream &_os) const {
-    _os << Internal[0]; // VM is at least garantee to be one
+    _os << Internal[0]; // VM is at least guarantee to be one
     for(int i=1; i<VMToLength<TVM>::Length; ++i)
       _os << "," << Internal[i];
     }
@@ -100,6 +124,29 @@ public:
       }
   }
 
+  DataElement GetAsDataElement() const {
+    DataElement ret;
+    std::ostringstream os;
+    EncodingImplementation<VRToEncoding<TVR>::Mode>::Write(Internal,
+      GetLength(),os);
+    ret.SetVR( (VR::VRType)TVR );
+    assert( ret.GetVR() != VR::SQ );
+    if( (VR::VRType)VRToEncoding<TVR>::Mode == VR::VRASCII )
+      {
+      if( GetVR() != VR::UI )
+        {
+        if( os.str().size() % 2 )
+          {
+          os << " ";
+          }
+        }
+      }
+    VL::Type osStrSize = (VL::Type)os.str().size();
+    ret.SetByteValue( os.str().c_str(), osStrSize );
+
+    return ret;
+  }
+
   void Read(std::istream &_is) {
     return EncodingImplementation<VRToEncoding<TVR>::Mode>::Read(Internal,
       GetLength(),_is);
@@ -189,10 +236,10 @@ public:
     char sep;
     //std::cout << "GetLength: " << af->GetLength() << std::endl;
     for(unsigned long i=1; i<length;++i) {
-      assert( _is );
+      //assert( _is );
       // Get the separator in between the values
       _is >> std::ws >> sep; //_is.get(sep);
-      assert( sep == '\\' ); // FIXME: Bad use of assert
+      //assert( sep == '\\' ); // FIXME: Bad use of assert
       _is >> std::ws >> data[i];
       }
     }
@@ -216,42 +263,182 @@ public:
     }
 };
 
+//#define VRDS16ILLEGAL
+
+#ifdef VRDS16ILLEGAL
 template < typename Float >
 std::string to_string ( Float data ) {
   std::stringstream in;
   // in.imbue(std::locale::classic()); // This is not required AFAIK
-  unsigned long const digits =
-    static_cast< unsigned long >(
+  int const digits =
+    static_cast< int >(
     - std::log( std::numeric_limits<Float>::epsilon() )
-    / std::log( 10.0 ) );
+    / static_cast< Float >( std::log( 10.0 ) ) );
   if ( in << std::dec << std::setprecision(/*2+*/digits) << data ) {
     return ( in.str() );
   } else {
     throw "Impossible Conversion"; // should not happen ...
   }
 }
+#else
+// http://stackoverflow.com/questions/32631178/writing-ieee-754-1985-double-as-ascii-on-a-limited-16-bytes-string
 
-/* Writing VR::DS is not that easy after all */
-// http://groups.google.com/group/comp.lang.c++/browse_thread/thread/69ccd26f000a0802
-template<> inline void EncodingImplementation<VR::VRASCII>::Write(const float * data, unsigned long length, std::ostream &_os)  {
-    assert( data );
-    assert( length );
-    assert( _os );
-    _os << to_string(data[0]);
-    for(unsigned long i=1; i<length; ++i) {
-      assert( _os );
-      _os << "\\" << to_string(data[i]);
-      }
+static inline void clean(char *mant) {
+  char *ix = mant + strlen(mant) - 1;
+  while(('0' == *ix) && (ix > mant)) {
+    *ix-- = '\0';
+  }
+  if ('.' == *ix) {
+    *ix = '\0';
+  }
+}
+
+static int add1(char *buf, int n) {
+  if (n < 0) return 1;
+  if (buf[n] == '9') {
+    buf[n] = '0';
+    return add1(buf, n-1);
+  }
+  else {
+    buf[n] = (char)(buf[n] + 1);
+  }
+  return 0;
+}
+
+static int doround(char *buf, unsigned int n) {
+  char c;
+  if (n >= strlen(buf)) return 0;
+  c = buf[n];
+  buf[n] = 0;
+  if ((c >= '5') && (c <= '9')) return add1(buf, n-1);
+  return 0;
+}
+
+#if defined(_MSC_VER) && (_MSC_VER < 1900)
+#define snprintf _snprintf
+#endif
+
+static int roundat(char *buf, size_t bufLen, unsigned int i, int iexp) {
+  if (doround(buf, i) != 0) {
+    iexp += 1;
+    switch(iexp) {
+    case -2:
+      strcpy(buf, ".01");
+      break;
+    case -1:
+      strcpy(buf, ".1");
+      break;
+    case 0:
+      strcpy(buf, "1.");
+      break;
+    case 1:
+      strcpy(buf, "10");
+      break;
+    case 2:
+      strcpy(buf, "100");
+      break;
+    default:
+      snprintf(buf, bufLen, "1e%d", iexp);
     }
+    return 1;
+  }
+  return 0;
+}
+
+template < typename Float >
+static void x16printf(char *buf, int size, Float f) {
+  char line[40];
+  char *mant = line + 1;
+  int iexp, lexp, i;
+  char exp[6];
+
+  if (f < 0) {
+    f = -f;
+    size -= 1;
+    *buf++ = '-';
+  }
+  snprintf(line, sizeof(line), "%1.16e", f);
+  if (line[0] == '-') {
+    f = -f;
+    size -= 1;
+    *buf++ = '-';
+    snprintf(line, sizeof(line), "%1.16e", f);
+  }
+  *mant = line[0];
+  i = (int)strcspn(mant, "eE");
+  mant[i] = '\0';
+  iexp = (int)strtol(mant + i + 1, nullptr, 10);
+  lexp = snprintf(exp, sizeof(exp), "e%d", iexp);
+  if ((iexp >= size) || (iexp < -3)) {
+    i = roundat(mant, sizeof(line) - 1, size - 1 -lexp, iexp);
+    if(i == 1) {
+      strcpy(buf, mant);
+      return;
+    }
+    buf[0] = mant[0];
+    buf[1] = '.';
+    strncpy(buf + i + 2, mant + 1, size - 2 - lexp);
+    buf[size-lexp] = 0;
+    clean(buf);
+    strcat(buf, exp);
+  }
+  else if (iexp >= size - 2) {
+    roundat(mant, sizeof(line) - 1, iexp + 1, iexp);
+    strcpy(buf, mant);
+  }
+  else if (iexp >= 0) {
+    i = roundat(mant, sizeof(line) - 1, size - 1, iexp);
+    if (i == 1) {
+      strcpy(buf, mant);
+      return;
+    }
+    strncpy(buf, mant, iexp + 1);
+    buf[iexp + 1] = '.';
+    strncpy(buf + iexp + 2, mant + iexp + 1, size - iexp - 1);
+    buf[size] = 0;
+    clean(buf);
+  }
+  else {
+    int j;
+    i = roundat(mant, sizeof(line) - 1, size + 1 + iexp, iexp);
+    if (i == 1) {
+      strcpy(buf, mant);
+      return;
+    }
+    buf[0] = '.';
+    for(j=0; j< -1 - iexp; j++) {
+      buf[j+1] = '0';
+    }
+    strncpy(buf - iexp, mant, size + 1 + iexp);
+    buf[size] = 0;
+    clean(buf);
+  }
+}
+#if defined(_MSC_VER) && (_MSC_VER < 1900)
+#undef snprintf
+#endif
+
+#endif
 
 template<> inline void EncodingImplementation<VR::VRASCII>::Write(const double* data, unsigned long length, std::ostream &_os)  {
     assert( data );
     assert( length );
     assert( _os );
+#ifdef VRDS16ILLEGAL
     _os << to_string(data[0]);
+#else
+    char buf[16+1];
+    x16printf(buf, 16, data[0]);
+    _os << buf;
+#endif
     for(unsigned long i=1; i<length; ++i) {
       assert( _os );
+#ifdef VRDS16ILLEGAL
       _os << "\\" << to_string(data[i]);
+#else
+      x16printf(buf, 16, data[i]);
+      _os << "\\" << buf;
+#endif
       }
     }
 
@@ -331,7 +518,7 @@ public:
 };
 
 // For particular case for ASCII string
-// WARNING: This template explicitely instanciates a particular
+// WARNING: This template explicitly instantiates a particular
 // EncodingImplementation THEREFORE it is required to be declared after the
 // EncodingImplementation is needs (doh!)
 #if 0
@@ -363,7 +550,7 @@ public:
   }
   // Implementation of Print is common to all Mode (ASCII/Binary)
   void Print(std::ostream &_os) const {
-    _os << Internal[0]; // VM is at least garantee to be one
+    _os << Internal[0]; // VM is at least guarantee to be one
     for(int i=1; i<VMToLength<TVM>::Length; ++i)
       _os << "," << Internal[i];
     }
@@ -381,23 +568,28 @@ private:
 template< int TVM>
 class Element<VR::PN, TVM> : public StringElement<TVM>
 {
+  enum { ElementDisableCombinationsCheck = sizeof ( ElementDisableCombinations<VR::PN, TVM> ) };
 };
 #endif
 
 // Implementation for the undefined length (dynamically allocated array)
-template<int TVR>
+template<long long TVR>
 class Element<TVR, VM::VM1_n>
 {
+  enum { ElementDisableCombinationsCheck = sizeof ( ElementDisableCombinations<TVR, VM::VM1_n> ) };
 public:
   // This the way to prevent default initialization
-  explicit Element() { Internal=0; Length=0; Save = false; }
+  explicit Element() { Internal=nullptr; Length=0; Save = false; }
   ~Element() {
     if( Save ) {
       delete[] Internal;
     }
-    Internal = 0;
+    Internal = nullptr;
   }
 
+  static VR  GetVR()  { return (VR::VRType)TVR; }
+  static VM  GetVM()  { return VM::VM1_n; }
+
   // Length manipulation
   // SetLength should really be protected anyway...all operation
   // should go through SetArray
@@ -436,13 +628,13 @@ public:
     else {
       // TODO rewrite this stupid code:
       assert( Length == 0 );
-      assert( Internal == 0 );
+      assert( Internal == nullptr );
       assert( Save == false );
       Length = len / sizeof(Type);
       //assert( (len / sizeof(Type)) * sizeof(Type) == len );
       // MR00010001.dcm is a tough kid: 0019,105a is supposed to be VR::FL, VM::VM3 but
       // length is 14 bytes instead of 12 bytes. Simply consider value is total garbage.
-      if( (len / sizeof(Type)) * sizeof(Type) != len ) { Internal = 0; Length = 0; }
+      if( (len / sizeof(Type)) * sizeof(Type) != len ) { Internal = nullptr; Length = 0; }
       else Internal = const_cast<Type*>(array);
       }
       Save = save;
@@ -456,7 +648,7 @@ public:
     return Internal[idx];
   }
   typename VRToType<TVR>::Type &GetValue(unsigned int idx = 0) {
-    assert( idx < Length );
+    //assert( idx < Length );
     return Internal[idx];
   }
   typename VRToType<TVR>::Type operator[] (unsigned int idx) const {
@@ -467,10 +659,10 @@ public:
     assert( bv ); // That would be bad...
     if( (VR::VRType)(VRToEncoding<TVR>::Mode) == VR::VRBINARY )
       {
-      const Type* array = (Type*)bv->GetPointer();
+      const Type* array = (const Type*)bv->GetVoidPointer();
       if( array ) {
         assert( array ); // That would be bad...
-        assert( Internal == 0 );
+        assert( Internal == nullptr );
         SetArray(array, bv->GetLength() ); }
       }
     else
@@ -509,7 +701,7 @@ public:
   void Print(std::ostream &_os) const {
     assert( Length );
     assert( Internal );
-    _os << Internal[0]; // VM is at least garantee to be one
+    _os << Internal[0]; // VM is at least guarantee to be one
     const unsigned long length = GetLength() < 25 ? GetLength() : 25;
     for(unsigned long i=1; i<length; ++i)
       _os << "," << Internal[i];
@@ -532,12 +724,24 @@ public:
   DataElement GetAsDataElement() const {
     DataElement ret;
     ret.SetVR( (VR::VRType)TVR );
+    assert( ret.GetVR() != VR::SQ );
     if( Internal )
       {
       std::ostringstream os;
       EncodingImplementation<VRToEncoding<TVR>::Mode>::Write(Internal,
         GetLength(),os);
-      ret.SetByteValue( os.str().c_str(), os.str().size() );
+      if( (VR::VRType)VRToEncoding<TVR>::Mode == VR::VRASCII )
+        {
+        if( GetVR() != VR::UI )
+          {
+          if( os.str().size() % 2 )
+            {
+            os << " ";
+            }
+          }
+        }
+      VL::Type osStrSize = (VL::Type)os.str().size();
+      ret.SetByteValue( os.str().c_str(), osStrSize );
       }
     return ret;
   }
@@ -560,10 +764,10 @@ protected:
     assert( bv ); // That would be bad...
     if( (VR::VRType)(VRToEncoding<TVR>::Mode) == VR::VRBINARY )
       {
-      const Type* array = (Type*)bv->GetPointer();
+      const Type* array = (const Type*)bv->GetPointer();
       if( array ) {
         assert( array ); // That would be bad...
-        assert( Internal == 0 );
+        assert( Internal == nullptr );
         SetArray(array, bv->GetLength() ); }
       }
     else
@@ -586,9 +790,20 @@ private:
 //class Element<VR::OB, TVM > : public Element<VR::OB, VM::VM1_n> {};
 
 // Partial specialization for derivatives of 1-n : 2-n, 3-n ...
-template<int TVR>
+template<long long TVR>
+class Element<TVR, VM::VM1_2> : public Element<TVR, VM::VM1_n>
+{
+public:
+  typedef Element<TVR, VM::VM1_n> Parent;
+  void SetLength(int len) {
+    if( len != 1 && len != 2 ) return;
+    Parent::SetLength(len);
+  }
+};
+template<long long TVR>
 class Element<TVR, VM::VM2_n> : public Element<TVR, VM::VM1_n>
 {
+  enum { ElementDisableCombinationsCheck = sizeof ( ElementDisableCombinations<TVR, VM::VM2_n> ) };
 public:
   typedef Element<TVR, VM::VM1_n> Parent;
   void SetLength(int len) {
@@ -596,9 +811,10 @@ public:
     Parent::SetLength(len);
   }
 };
-template<int TVR>
+template<long long TVR>
 class Element<TVR, VM::VM2_2n> : public Element<TVR, VM::VM2_n>
 {
+  enum { ElementDisableCombinationsCheck = sizeof ( ElementDisableCombinations<TVR, VM::VM2_2n> ) };
 public:
   typedef Element<TVR, VM::VM2_n> Parent;
   void SetLength(int len) {
@@ -606,9 +822,10 @@ public:
     Parent::SetLength(len);
   }
 };
-template<int TVR>
+template<long long TVR>
 class Element<TVR, VM::VM3_n> : public Element<TVR, VM::VM1_n>
 {
+  enum { ElementDisableCombinationsCheck = sizeof ( ElementDisableCombinations<TVR, VM::VM3_n> ) };
 public:
   typedef Element<TVR, VM::VM1_n> Parent;
   void SetLength(int len) {
@@ -616,9 +833,10 @@ public:
     Parent::SetLength(len);
   }
 };
-template<int TVR>
+template<long long TVR>
 class Element<TVR, VM::VM3_3n> : public Element<TVR, VM::VM3_n>
 {
+  enum { ElementDisableCombinationsCheck = sizeof ( ElementDisableCombinations<TVR, VM::VM3_3n> ) };
 public:
   typedef Element<TVR, VM::VM3_n> Parent;
   void SetLength(int len) {
@@ -626,6 +844,16 @@ public:
     Parent::SetLength(len);
   }
 };
+template<long long TVR>
+class Element<TVR, VM::VM3_4> : public Element<TVR, VM::VM1_n>
+{
+public:
+  typedef Element<TVR, VM::VM1_n> Parent;
+  void SetLength(int len) {
+    if( len != 3 && len != 4 ) return;
+    Parent::SetLength(len);
+  }
+};
 
 
 //template<int T> struct VRToLength;
@@ -634,12 +862,13 @@ public:
 //template<>
 //class Element<VR::AS> : public Element<VR::AS, VRToLength<VR::AS>::Length >
 
-// only 0010 1010 AS 1 Patient’s Age
+// only 0010 1010 AS 1 Patient's Age
 template<>
 class Element<VR::AS, VM::VM5>
 {
+  enum { ElementDisableCombinationsCheck = sizeof ( ElementDisableCombinations<VR::AS, VM::VM5> ) };
 public:
-  char Internal[VMToLength<VM::VM5>::Length];
+  char Internal[VMToLength<VM::VM5>::Length * sizeof( VRToType<VR::AS>::Type )];
   void Print(std::ostream &_os) const {
     _os << Internal;
     }
@@ -648,17 +877,15 @@ public:
   }
 };
 
+
 template <>
 class Element<VR::OB, VM::VM1> : public Element<VR::OB, VM::VM1_n> {};
-// Make it impossible to compile any other cases:
-template <int TVM> class Element<VR::OB, TVM>;
 
 // Same for OW:
 template <>
 class Element<VR::OW, VM::VM1> : public Element<VR::OW, VM::VM1_n> {};
-// Make it impossible to compile any other cases:
-template <int TVM> class Element<VR::OW, TVM>;
 
-} // namespace gdcm
+
+} // namespace gdcm_ns
 
 #endif //GDCMELEMENT_H
