@@ -1,9 +1,8 @@
 /*=========================================================================
 
   Program: GDCM (Grassroots DICOM). A DICOM library
-  Module:  $URL$
 
-  Copyright (c) 2006-2010 Mathieu Malaterre
+  Copyright (c) 2006-2011 Mathieu Malaterre
   All rights reserved.
   See Copyright.txt or http://gdcm.sourceforge.net/Copyright.html for details.
 
@@ -12,7 +11,6 @@
      PURPOSE.  See the above copyright notice for more information.
 
 =========================================================================*/
-
 #ifndef GDCMPREAMBLE_H
 #define GDCMPREAMBLE_H
 
@@ -33,47 +31,51 @@ public:
 
   friend std::ostream &operator<<(std::ostream &_os, const Preamble &_val);
 
-  // Clear
+  /// Clear
   void Clear();
 
-  // Set Preamble to the default one
+  /// Set Preamble to the default one
   void Valid();
   void Create();
   void Remove();
 
- // Read
+  /// Read Preamble
   std::istream &Read(std::istream &is);
 
-  // Write
+  /// Write Preamble
   std::ostream const &Write(std::ostream &os) const;
 
+  /// Print Preamble
   void Print(std::ostream &os) const;
 
+  /// Get internal pointer to preamble
   const char *GetInternal() const { return Internal; }
 
+  /// Check if Preamble is empty
   bool IsEmpty() const { return !Internal; }
 
+  /// Return size of Preamble
   VL GetLength() const { return 128 + 4; }
 
-  Preamble(Preamble const &)
+  Preamble(Preamble const &):Internal(nullptr)
     {
     Create();
     }
   Preamble& operator=(Preamble const &)
     {
+    Internal = nullptr;
     Create();
     return *this;
     }
 protected:
   //
-  bool IsValid() const { 
+  bool IsValid() const {
     // is (IsValid == true) => Internal was read
-    return true; 
+    return true;
   }
 
- 
 private:
-  char *Internal; 
+  char *Internal;
 };
 //-----------------------------------------------------------------------------
 inline std::ostream& operator<<(std::ostream &os, const Preamble &val)
@@ -82,8 +84,6 @@ inline std::ostream& operator<<(std::ostream &os, const Preamble &val)
   return os;
 }
 
-
 } // end namespace gdcm
 
 #endif //GDCMPREAMBLE_H
-
