@@ -1,9 +1,8 @@
 /*=========================================================================
 
   Program: GDCM (Grassroots DICOM). A DICOM library
-  Module:  $URL$
 
-  Copyright (c) 2006-2010 Mathieu Malaterre
+  Copyright (c) 2006-2011 Mathieu Malaterre
   All rights reserved.
   See Copyright.txt or http://gdcm.sourceforge.net/Copyright.html for details.
 
@@ -28,8 +27,7 @@ namespace gdcm
 class GDCM_EXPORT Patient
 {
 public:
-  Patient() { 
-  }
+  Patient() = default;
 
 private:
   std::vector<Study> StudyList;
