@@ -1,9 +1,8 @@
 /*=========================================================================
 
   Program: GDCM (Grassroots DICOM). A DICOM library
-  Module:  $URL$
 
-  Copyright (c) 2006-2010 Mathieu Malaterre
+  Copyright (c) 2006-2011 Mathieu Malaterre
   All rights reserved.
   See Copyright.txt or http://gdcm.sourceforge.net/Copyright.html for details.
 
@@ -14,35 +13,33 @@
 =========================================================================*/
 #include "gdcmDirectionCosines.h"
 
+#include <cmath> // fabs
+#include <cstdio> // sscanf
 #include <limits>
-#include <math.h> // fabs
-#include <stdio.h> // sscanf
 
 namespace gdcm
 {
 
 DirectionCosines::DirectionCosines()
 {
-    Values[0] = 1;
-    Values[1] = 0;
-    Values[2] = 0;
-    Values[3] = 0;
-    Values[4] = 1;
-    Values[5] = 0;
+  Values[0] = 1;
+  Values[1] = 0;
+  Values[2] = 0;
+  Values[3] = 0;
+  Values[4] = 1;
+  Values[5] = 0;
 }
 
 DirectionCosines::DirectionCosines(const double dircos[6])
 {
-    Values[0] = dircos[0];
-    Values[1] = dircos[1];
-    Values[2] = dircos[2];
-    Values[3] = dircos[3];
-    Values[4] = dircos[4];
-    Values[5] = dircos[5];
+  Values[0] = dircos[0];
+  Values[1] = dircos[1];
+  Values[2] = dircos[2];
+  Values[3] = dircos[3];
+  Values[4] = dircos[4];
+  Values[5] = dircos[5];
 }
 
-DirectionCosines::~DirectionCosines() {}
-
 void DirectionCosines::Print(std::ostream &os) const
 {
   os << Values[0] << ",";
@@ -62,7 +59,7 @@ bool DirectionCosines::IsValid() const
   double norm_v1 = Values[0] * Values[0] + Values[1]*Values[1] + Values[2]*Values[2];
   double norm_v2 = Values[3] * Values[3] + Values[4]*Values[4] + Values[5]*Values[5];
   double dot = Dot();
-  
+
   bool ret = false;
   if( fabs(norm_v1 - 1) < epsilon && fabs(norm_v2 - 1) < epsilon )
     {
@@ -85,19 +82,39 @@ void DirectionCosines::Cross(double z[3]) const
   z[0] = Zx; z[1] = Zy; z[2] = Zz;
 }
 
-double DirectionCosines::Dot() const
+static inline double DotImpl(const double x[3], const double y[3])
 {
-  const double *x = Values;
-  const double *y = x+3;
   return x[0]*y[0] + x[1]*y[1] + x[2]*y[2];
 }
 
+double DirectionCosines::Dot(const double x[3], const double y[3])
+{
+  return DotImpl(x, y);
+}
+
+double DirectionCosines::Dot() const
+{
+  return DotImpl(Values, Values+3);
+}
+
 // static function is within gdcm:: namespace, so should not pollute too much on UNIX
-static double Norm(const double x[3])
+static inline double Norm(const double x[3])
 {
   return sqrt(x[0]*x[0] + x[1]*x[1] + x[2]*x[2]);
 }
 
+void DirectionCosines::Normalize(double v[3])
+{
+  double den;
+  if ( (den = Norm(v)) != 0.0 )
+    {
+    for (int i=0; i < 3; i++)
+      {
+      v[i] /= den;
+      }
+    }
+}
+
 void DirectionCosines::Normalize()
 {
   double *x = Values;
@@ -121,11 +138,14 @@ void DirectionCosines::Normalize()
 
 bool DirectionCosines::SetFromString(const char *str)
 {
-  if( !str ) return false;
-  int n = sscanf( str, "%lf\\%lf\\%lf\\%lf\\%lf\\%lf", Values, Values+1, Values+2, Values+3, Values+4, Values+5 );
-  if( n == 6 )
+  if( str )
     {
-    return true;
+    const int n = sscanf( str, R"(%lf\%lf\%lf\%lf\%lf\%lf)",
+      Values, Values+1, Values+2, Values+3, Values+4, Values+5 );
+    if( n == 6 )
+      {
+      return true;
+      }
     }
   // else
   Values[0] = 1;
@@ -158,6 +178,4 @@ double DirectionCosines::ComputeDistAlongNormal(const double ipp[3]) const
   return dist;
 }
 
-
-}
-
+} // end namespace gdcm
