@@ -1,9 +1,8 @@
 /*=========================================================================
 
   Program: GDCM (Grassroots DICOM). A DICOM library
-  Module:  $URL$
 
-  Copyright (c) 2006-2010 Mathieu Malaterre
+  Copyright (c) 2006-2011 Mathieu Malaterre
   All rights reserved.
   See Copyright.txt or http://gdcm.sourceforge.net/Copyright.html for details.
 
@@ -23,24 +22,29 @@ namespace gdcm
 class DataElement;
 /**
  * \brief ImageApplyLookupTable class
- * It applies the LUT the PixelData (only PALETTE_COLOR images)
+ * \details It applies the LUT the PixelData (only PALETTE_COLOR images)
  * Output will be a PhotometricInterpretation=RGB image
  */
 class GDCM_EXPORT ImageApplyLookupTable : public ImageToImageFilter
 {
 public:
-  ImageApplyLookupTable() {}
-  ~ImageApplyLookupTable() {}
+  ImageApplyLookupTable();
+  ~ImageApplyLookupTable();
 
   /// Apply
   bool Apply();
 
+  /// RGB8 ?
+  void SetRGB8(bool b);
+
 protected:
 
 private:
+  struct impl;
+  // PIMPL idiom
+  impl* pimpl;
 };
 
 } // end namespace gdcm
 
 #endif //GDCMIMAGEAPPLYLOOKUPTABLE_H
-
