@@ -1,9 +1,8 @@
 /*=========================================================================
 
   Program: GDCM (Grassroots DICOM). A DICOM library
-  Module:  $URL$
 
-  Copyright (c) 2006-2010 Mathieu Malaterre
+  Copyright (c) 2006-2011 Mathieu Malaterre
   All rights reserved.
   See Copyright.txt or http://gdcm.sourceforge.net/Copyright.html for details.
 
@@ -23,14 +22,14 @@ namespace gdcm
 class DataElement;
 /**
  * \brief ImageChangePlanarConfiguration class
- * Class to change the Planar configuration of an input DICOM
- * By default it will change into the more usual reprensentation: PlanarConfiguration = 0
+ * \details Class to change the Planar configuration of an input DICOM
+ * By default it will change into the more usual representation: PlanarConfiguration = 0
  */
 class GDCM_EXPORT ImageChangePlanarConfiguration : public ImageToImageFilter
 {
 public:
   ImageChangePlanarConfiguration():PlanarConfiguration(0) {}
-  ~ImageChangePlanarConfiguration() {}
+  ~ImageChangePlanarConfiguration() = default;
 
   /// Set/Get requested PlanarConfigation
   void SetPlanarConfiguration(unsigned int pc) { PlanarConfiguration = pc; }
@@ -67,7 +66,7 @@ size_t ImageChangePlanarConfiguration::RGBPlanesToRGBPixels(T *out, const T *r,
     *pout++ = *b++;
     }
 
-  assert( (size_t)(pout - out) == 3 * s * sizeof(T) );
+  assert( (size_t)(pout - out) == 3 * s );
   return pout - out;
 }
 
@@ -81,7 +80,7 @@ size_t ImageChangePlanarConfiguration::RGBPixelsToRGBPlanes(T *r, T *g, T *b, co
     *g++ = *prgb++;
     *b++ = *prgb++;
     }
-  assert( (size_t)(prgb - rgb) == 3 * s * sizeof(T) );
+  assert( (size_t)(prgb - rgb) == 3 * s );
   return prgb - rgb;
 }
 
@@ -89,4 +88,3 @@ size_t ImageChangePlanarConfiguration::RGBPixelsToRGBPlanes(T *r, T *g, T *b, co
 } // end namespace gdcm
 
 #endif //GDCMIMAGECHANGEPLANARCONFIGURATION_H
-
