@@ -21,8 +21,8 @@
 #include <iomanip>
 #include <iterator>
 
+#include <climits>
 #include <cstring>
-#include <limits.h>
 
 namespace gdcm
 {
@@ -157,14 +157,13 @@ assert(0); // Do not use this code !
   const unsigned char *a = copy + 0;
   const unsigned char *b = copy + size;
   const unsigned char *c = copy + size + size;
-  int R, G, B;
 
   unsigned char *p = (unsigned char*)dummy_buffer;
   for (unsigned long j = 0; j < size; ++j)
     {
-    R = 38142 *(*a-16) + 52298 *(*c -128);
-    G = 38142 *(*a-16) - 26640 *(*c -128) - 12845 *(*b -128);
-    B = 38142 *(*a-16) + 66093 *(*b -128);
+    int R = 38142 *(*a-16) + 52298 *(*c -128);
+    int G = 38142 *(*a-16) - 26640 *(*c -128) - 12845 *(*b -128);
+    int B = 38142 *(*a-16) + 66093 *(*b -128);
 
     R = (R+16384)>>15;
     G = (G+16384)>>15;
@@ -461,8 +460,7 @@ bool ImageCodec::CleanupUnusedBits(char * data8, size_t datalen)
       smask = (uint16_t)(
         smask << ( 16 - (PF.GetBitsAllocated() - PF.GetBitsStored() + 1) ));
       // nmask : to propagate sign bit on negative values
-      int16_t nmask = (int16_t)0x8000;
-      nmask = (int16_t)(nmask >> ( PF.GetBitsAllocated() - PF.GetBitsStored() - 1 ));
+      int16_t nmask = (int16_t)(0x8000U >> ( PF.GetBitsAllocated() - PF.GetBitsStored() - 1 ));
 
       uint16_t *start = (uint16_t*)data;
       for( uint16_t *p = start ; p != start + datalen / 2; ++p )
@@ -517,8 +515,7 @@ bool ImageCodec::DoOverlayCleanup(std::istream &is, std::ostream &os)
       smask = (uint16_t)(
         smask << ( 16 - (PF.GetBitsAllocated() - PF.GetBitsStored() + 1) ));
       // nmask : to propagate sign bit on negative values
-      int16_t nmask = (int16_t)0x8000;
-      nmask = (int16_t)(nmask >> ( PF.GetBitsAllocated() - PF.GetBitsStored() - 1 ));
+      int16_t nmask = (int16_t)(0x8000U >> ( PF.GetBitsAllocated() - PF.GetBitsStored() - 1 ));
 
       uint16_t c;
       while( is.read((char*)&c,2) )
@@ -538,14 +535,22 @@ bool ImageCodec::DoOverlayCleanup(std::istream &is, std::ostream &os)
     else // Pixel are unsigned
       {
 #if 1
-      uint16_t c;
-      while( is.read((char*)&c,2) )
+      // On Windows, is.read and os.write are expensive operations.
+      // If we called it for each value then conversion would be extremely slow.
+      // Therefore we read/mask/write 1000 values at once.
+      const unsigned int bufferSize = 1000;
+      std::vector<uint16_t> buffer(bufferSize);
+      while (is)
         {
-        c = (uint16_t)(
-          (c >> (PF.GetBitsStored() - PF.GetHighBit() - 1)) & pmask);
-        os.write((char*)&c, 2 );
+        is.read((char *)buffer.data(), bufferSize * sizeof(uint16_t));
+        std::streamsize bytesRead = is.gcount();
+        std::vector<uint16_t>::iterator validBufferEnd = buffer.begin() + bytesRead / sizeof(uint16_t);
+        for (std::vector<uint16_t>::iterator it = buffer.begin(); it != validBufferEnd; ++it)
+          {
+          *it = ((*it >> (PF.GetBitsStored() - PF.GetHighBit() - 1)) & pmask);
+          }
+        os.write((char *)buffer.data(), bytesRead);
         }
-      //os.rdbuf( is.rdbuf() );
 #else
       //std::ostreambuf_iterator<char> end_of_stream_iterator;
       //std::ostreambuf_iterator<char> out_iter(os.rdbuf());
@@ -585,7 +590,7 @@ bool ImageCodec::DecodeByStreams(std::istream &is, std::ostream &os)
   assert( PI != PhotometricInterpretation::UNKNOWN );
   std::stringstream bs_os; // ByteSwap
   std::stringstream pcpc_os; // Padded Composite Pixel Code
-  std::stringstream pi_os; // PhotometricInterpretation
+  //std::stringstream pi_os; // PhotometricInterpretation
   std::stringstream pl_os; // PlanarConf
   std::istream *cur_is = &is;
 
@@ -637,6 +642,7 @@ bool ImageCodec::DecodeByStreams(std::istream &is, std::ostream &os)
     // Nothing needs to be done
     break;
   case PhotometricInterpretation::YBR_FULL_422:
+  case PhotometricInterpretation::YBR_PARTIAL_422:
       {
       // US-GE-4AICL142.dcm
       // Hopefully it has been done by the JPEG decoder itself...
@@ -657,7 +663,6 @@ bool ImageCodec::DecodeByStreams(std::istream &is, std::ostream &os)
   default:
     gdcmErrorMacro( "Unhandled PhotometricInterpretation: " << PI );
     return false;
-    assert(0);
     }
 
   if( /*PlanarConfiguration ||*/ RequestPlanarConfiguration )
