@@ -1,9 +1,8 @@
 /*=========================================================================
 
   Program: GDCM (Grassroots DICOM). A DICOM library
-  Module:  $URL$
 
-  Copyright (c) 2006-2010 Mathieu Malaterre
+  Copyright (c) 2006-2011 Mathieu Malaterre
   All rights reserved.
   See Copyright.txt or http://gdcm.sourceforge.net/Copyright.html for details.
 
@@ -25,9 +24,9 @@ class Image;
 /**
  * \brief Image Converter
  * \note
- * This is the class used to convert from on gdcm::Image to another
- * This is typically used to convert let say YBR JPEG compressed 
- * gdcm::Image to a RAW RGB gdcm::Image. So that the buffer can be directly
+ * This is the class used to convert from on Image to another
+ * This is typically used to convert let say YBR JPEG compressed
+ * Image to a RAW RGB Image. So that the buffer can be directly
  * pass to third party application.
  * This filter is application level and not integrated directly in GDCM
  */
@@ -50,4 +49,3 @@ private:
 } // end namespace gdcm
 
 #endif //GDCMIMAGECONVERTER_H
-
