@@ -1,9 +1,8 @@
 /*=========================================================================
 
   Program: GDCM (Grassroots DICOM). A DICOM library
-  Module:  $URL$
 
-  Copyright (c) 2006-2010 Mathieu Malaterre
+  Copyright (c) 2006-2011 Mathieu Malaterre
   All rights reserved.
   See Copyright.txt or http://gdcm.sourceforge.net/Copyright.html for details.
 
@@ -16,8 +15,11 @@
 
 #include "gdcm_ljpeg16.h"
 
-#include <setjmp.h>
+#include <csetjmp>
 
 #define JPEGBITSCodec JPEG16Codec
-#include "gdcmJPEGBITSCodec.cxx"
-
+#define my_error_mgr my_error_mgr_16BIT
+#define JPEGInternals JPEGInternals_16BIT
+#define my_source_mgr my_source_mgr_16BIT
+#define my_destination_mgr my_destination_mgr_16BIT
+#include "gdcmJPEGBITSCodec.hxx"
