@@ -21,15 +21,15 @@
 #include "gdcmJPEG12Codec.h"
 #include "gdcmJPEG16Codec.h"
 
+#include <cstring>
 #include <numeric>
-#include <string.h>
 
 namespace gdcm
 {
 
-JPEGCodec::JPEGCodec():BitSample(0),Lossless(true),Quality(100)
+JPEGCodec::JPEGCodec():BitSample(0)/*,Lossless(true)*/,Quality(100)
 {
-  Internal = NULL;
+  Internal = nullptr;
 }
 
 JPEGCodec::~JPEGCodec()
@@ -50,12 +50,12 @@ double JPEGCodec::GetQuality() const
 
 void JPEGCodec::SetLossless(bool l)
 {
-  Lossless = l;
+  LossyFlag = !l;
 }
 
 bool JPEGCodec::GetLossless() const
 {
-  return Lossless;
+  return !LossyFlag;
 }
 
 bool JPEGCodec::CanDecode(TransferSyntax const &ts) const
@@ -92,7 +92,7 @@ void JPEGCodec::SetPixelFormat(PixelFormat const &pt)
 
   // We have the choice to decide to use Bits Stored or Bits Allocated, however in the case of
   // such an image as: gdcmData/MR16BitsAllocated_8BitsStored.dcm we are required to use
-  // bits allocated to deal with the logic to decide withe the encoder
+  // bits allocated to deal with the logic to decide with the encoder
   SetBitSample( pt.GetBitsAllocated() );
   //SetBitSample( pt.GetBitsStored() );
 }
@@ -100,8 +100,7 @@ void JPEGCodec::SetPixelFormat(PixelFormat const &pt)
 void JPEGCodec::SetupJPEGBitCodec(int bit)
 {
   BitSample = bit;
-  delete Internal; Internal = NULL;
-  assert( Internal == NULL );
+  delete Internal; Internal = nullptr;
   // what should I do with those single bit images ?
   if ( BitSample <= 8 )
     {
@@ -122,9 +121,6 @@ void JPEGCodec::SetupJPEGBitCodec(int bit)
     {
     // gdcmNonImageData/RT/RTDOSE.dcm
     gdcmWarningMacro( "Cannot instantiate JPEG codec for bit sample: " << bit );
-    // Clearly make sure Internal will not be used
-    delete Internal;
-    Internal = NULL;
     }
 }
 
@@ -136,6 +132,8 @@ void JPEGCodec::SetBitSample(int bit)
     Internal->SetDimensions( this->GetDimensions() );
     Internal->SetPlanarConfiguration( this->GetPlanarConfiguration() );
     Internal->SetPhotometricInterpretation( this->GetPhotometricInterpretation() );
+    Internal->SetLossless( this->GetLossless() );
+    Internal->SetQuality( this->GetQuality() );
     Internal->ImageCodec::SetPixelFormat( this->ImageCodec::GetPixelFormat() );
     //Internal->SetNeedOverlayCleanup( this->AreOverlaysInPixelData() );
     }
@@ -157,7 +155,6 @@ bool JPEGCodec::Decode(DataElement const &in, DataElement &out)
   std::stringstream os;
   if( sf0 )
     {
-    //unsigned long pos = 0;
     for(unsigned int i = 0; i < sf0->GetNumberOfFragments(); ++i)
       {
       std::stringstream is;
@@ -174,7 +171,23 @@ bool JPEGCodec::Decode(DataElement const &in, DataElement &out)
       // PHILIPS_Gyroscan-12-MONO2-Jpeg_Lossless.dcm
       if( !r )
         {
-        return false;
+        gdcmDebugMacro( "Failed to decompress Frag #" << i );
+        const bool suspended = Internal->IsStateSuspension();
+        const size_t nfrags = sf0->GetNumberOfFragments();
+        // In case of chunked-jpeg, this is always an error
+        if( suspended )
+          return false;
+        // Ok so we are decoding a multiple frame jpeg DICOM file:
+        // if we are lucky, we might be trying to decode some sort of broken multi-frame
+        // DICOM file. In this case check that we have read all Fragment properly:
+        if( i >= this->GetDimensions()[2] )
+          {
+          // JPEGInvalidSecondFrag.dcm
+          assert( nfrags == this->GetNumberOfDimensions() ); (void)nfrags; // sentinel
+          gdcmWarningMacro( "Invalid JPEG Fragment found at pos #" << i + 1 << ". Skipping it" );
+          }
+        else
+          return false;
         }
       }
     }
@@ -196,7 +209,7 @@ bool JPEGCodec::Decode(DataElement const &in, DataElement &out)
       is0.seekg(0);
       SequenceOfFragments sf_bug;
       try {
-        sf_bug.Read<SwapperNoOp>(is0);
+        sf_bug.Read<SwapperNoOp>(is0,true);
       } catch ( ... ) {
         return false;
       }
@@ -224,9 +237,13 @@ bool JPEGCodec::Decode(DataElement const &in, DataElement &out)
       }
     }
   //assert( pos == len );
-  std::string str = os.str();
-  VL::Type strSize = (VL::Type)str.size();
-  out.SetByteValue( &str[0], strSize );
+  const size_t sizeOfOs = (size_t)os.tellp();
+  os.seekp( 0, std::ios::beg );
+  ByteValue * bv = new ByteValue;
+  bv->SetLength( (uint32_t)sizeOfOs );
+  bv->Read<SwapperNoOp>( os );
+  out.SetValue( *bv );
+
   return true;
 }
 
@@ -257,7 +274,7 @@ bool JPEGCodec::GetHeaderInfo( std::istream & is, TransferSyntax &ts )
       SetupJPEGBitCodec( Internal->BitSample );
       if( Internal && Internal->GetHeaderInfo(is, ts) )
         {
-        // Foward everything back to meta jpeg codec:
+        // Forward everything back to meta jpeg codec:
         this->SetLossyFlag( Internal->GetLossyFlag() );
         this->SetDimensions( Internal->GetDimensions() );
         this->SetPhotometricInterpretation( Internal->GetPhotometricInterpretation() );
@@ -276,7 +293,7 @@ bool JPEGCodec::GetHeaderInfo( std::istream & is, TransferSyntax &ts )
     return false;
     }
   // else
-  // Foward everything back to meta jpeg codec:
+  // Forward everything back to meta jpeg codec:
   this->SetLossyFlag( Internal->GetLossyFlag() );
   this->SetDimensions( Internal->GetDimensions() );
   this->SetPhotometricInterpretation( Internal->GetPhotometricInterpretation() );
@@ -303,6 +320,7 @@ bool JPEGCodec::Code(DataElement const &in, DataElement &out)
   //sq->GetTable().SetByteValue( dummy, sizeof(dummy) );
 
   const ByteValue *bv = in.GetByteValue();
+  if(!bv) return false; // broken DICOM file ?
   const unsigned int *dims = this->GetDimensions();
   const char *input = bv->GetPointer();
   unsigned long len = bv->GetLength();
@@ -324,11 +342,11 @@ bool JPEGCodec::Code(DataElement const &in, DataElement &out)
       }
 
     std::string str = os.str();
-    assert( str.size() );
+    assert( !str.empty() );
     Fragment frag;
     //frag.SetTag( itemStart );
     VL::Type strSize = (VL::Type)str.size();
-    frag.SetByteValue( &str[0], strSize );
+    frag.SetByteValue( str.data(), strSize );
     sq->AddFragment( frag );
 
     }
@@ -362,6 +380,7 @@ bool JPEGCodec::DecodeByStreams(std::istream &is, std::ostream &os)
       if( Internal )
         {
         //Internal->SetPixelFormat( this->GetPixelFormat() ); // FIXME
+        Internal->SetDimensions( this->GetDimensions() );
         Internal->SetPlanarConfiguration( this->GetPlanarConfiguration() ); // meaningless ?
         Internal->SetPhotometricInterpretation( this->GetPhotometricInterpretation() );
         if( Internal->DecodeByStreams(is,tmpos) )
@@ -444,12 +463,12 @@ bool JPEGCodec::DecodeExtent(
 
   if( NumberOfDimensions == 2 )
     {
-    char *dummy_buffer = NULL;
+    //char *dummy_buffer = NULL;
     std::vector<char> vdummybuffer;
     size_t buf_size = 0;
 
     const Tag seqDelItem(0xfffe,0xe0dd);
-    gdcm::Fragment frag;
+    Fragment frag;
     unsigned int nfrags = 0;
     try
       {
@@ -461,7 +480,7 @@ bool JPEGCodec::DecodeExtent(
         // update
         buf_size = fraglen + oldlen;
         vdummybuffer.resize( buf_size );
-        dummy_buffer = &vdummybuffer[0];
+        //dummy_buffer = &vdummybuffer[0];
         // read J2K
         is.read( &vdummybuffer[oldlen], fraglen );
         }
@@ -509,7 +528,7 @@ bool JPEGCodec::DecodeExtent(
     assert( zmin == 0 );
 
     std::stringstream iis;
-    iis.write( &vdummybuffer[0], vdummybuffer.size() );
+    iis.write( vdummybuffer.data(), vdummybuffer.size() );
     std::stringstream os;
     bool b = DecodeByStreams(iis,os);
     if(!b) return false;
@@ -523,7 +542,7 @@ bool JPEGCodec::DecodeExtent(
     std::istream *theStream = &os;
     std::vector<char> buffer1;
     buffer1.resize( rowsize*bytesPerPixel );
-    char *tmpBuffer1 = &buffer1[0];
+    char *tmpBuffer1 = buffer1.data();
     unsigned int y, z;
     std::streamoff theOffset;
     for (z = zmin; z <= zmax; ++z)
@@ -543,7 +562,7 @@ bool JPEGCodec::DecodeExtent(
   else if ( NumberOfDimensions == 3 )
     {
     const Tag seqDelItem(0xfffe,0xe0dd);
-    gdcm::Fragment frag;
+    Fragment frag;
     std::streamoff thestart = is.tellg();
     unsigned int numfrags = 0;
     std::vector< size_t > offsets;
@@ -551,8 +570,8 @@ bool JPEGCodec::DecodeExtent(
       {
       //std::streamoff relstart = is.tellg();
       //assert( relstart - thestart == 8 );
-      std::streamoff off = frag.GetVL();
-      offsets.push_back( off );
+      const std::streamoff off = frag.GetVL();
+      offsets.push_back( (size_t)off );
       is.seekg( off, std::ios::cur );
       ++numfrags;
       }
@@ -566,7 +585,7 @@ bool JPEGCodec::DecodeExtent(
 
     for( unsigned int z = zmin; z <= zmax; ++z )
       {
-      size_t curoffset = std::accumulate( offsets.begin(), offsets.begin() + z, 0 );
+      size_t curoffset = std::accumulate( offsets.begin(), offsets.begin() + z, size_t(0) );
       is.seekg( thestart + curoffset + 8 * z, std::ios::beg );
       is.seekg( 8, std::ios::cur );
 
@@ -590,7 +609,7 @@ bool JPEGCodec::DecodeExtent(
 
       std::vector<char> buffer1;
       buffer1.resize( rowsize*bytesPerPixel );
-      char *tmpBuffer1 = &buffer1[0];
+      char *tmpBuffer1 = buffer1.data();
       unsigned int y;
       std::streamoff theOffset;
       for (y = ymin; y <= ymax; ++y)
@@ -608,4 +627,61 @@ bool JPEGCodec::DecodeExtent(
   return true;
 }
 
+bool JPEGCodec::IsStateSuspension() const
+{
+  assert( 0 );
+  return false;
+}
+
+ImageCodec * JPEGCodec::Clone() const
+{
+  JPEGCodec *copy = new JPEGCodec;
+  ImageCodec &ic = *copy;
+  ic = *this;
+  assert( copy->PF == PF );
+  //copy->SetupJPEGBitCodec( BitSample );
+  copy->SetPixelFormat( GetPixelFormat() );
+  assert( copy->BitSample == BitSample || BitSample == 0 );
+  //copy->Lossless = Lossless;
+  copy->Quality = Quality;
+
+  return copy;
+}
+
+bool JPEGCodec::EncodeBuffer( std::ostream & out,
+    const char *inbuffer, size_t inlen)
+{
+  assert( Internal );
+  return Internal->EncodeBuffer(out, inbuffer, inlen);
+}
+
+bool JPEGCodec::StartEncode( std::ostream & )
+{
+  return true;
+}
+bool JPEGCodec::IsRowEncoder()
+{
+  return true;
+}
+bool JPEGCodec::IsFrameEncoder()
+{
+  assert(0);
+  return false;
+}
+bool JPEGCodec::AppendRowEncode( std::ostream & os, const char * data, size_t datalen)
+{
+  return EncodeBuffer(os, data, datalen );
+}
+// TODO: technically the frame encoder could use the row encoder when present
+// this could reduce code duplication
+bool JPEGCodec::AppendFrameEncode( std::ostream & , const char * , size_t )
+{
+  assert(0);
+  return false;
+}
+bool JPEGCodec::StopEncode( std::ostream & )
+{
+  return true;
+}
+
 } // end namespace gdcm
