@@ -31,8 +31,7 @@ KAKADUCodec::KAKADUCodec()
 }
 
 KAKADUCodec::~KAKADUCodec()
-{
-}
+= default;
 
 bool KAKADUCodec::CanDecode(TransferSyntax const &ts) const
 {
@@ -62,7 +61,7 @@ bool KAKADUCodec::Decode(DataElement const &in, DataElement &out)
   return false;
 #else
   // First thing creates a j2k file from the fragment:
-  const gdcm::SequenceOfFragments *sf = in.GetSequenceOfFragments();
+  const SequenceOfFragments *sf = in.GetSequenceOfFragments();
   if(!sf) return false;
 
   if( NumberOfDimensions == 2 )
@@ -86,7 +85,7 @@ bool KAKADUCodec::Decode(DataElement const &in, DataElement &out)
     sf->WriteBuffer(outfile);
     outfile.close(); // flush !
 
-    //gdcm::Filename fn( System::GetCurrentProcessFileName() );
+    //Filename fn( System::GetCurrentProcessFileName() );
     //std::string executable_path = fn.GetPath();
 #ifdef GDCM_USE_SYSTEM_KAKADU
     std::string kakadu_command = GDCM_KAKADU_EXPAND_EXECUTABLE;
@@ -105,13 +104,13 @@ bool KAKADUCodec::Decode(DataElement const &in, DataElement &out)
     int ret = system(kakadu_command.c_str());
     //std::cerr << "system: " << ret << std::endl;
 
-    size_t len = gdcm::System::FileSize(output.c_str());
+    size_t len = System::FileSize(output.c_str());
     if(!len) return false;
 
-    std::ifstream is(output.c_str());
+    std::ifstream is(output.c_str(), std::ios::binary);
     char * buf = new char[len];
     is.read(buf, len);
-    out.SetTag( gdcm::Tag(0x7fe0,0x0010) );
+    out.SetTag( Tag(0x7fe0,0x0010) );
     out.SetByteValue( buf, len );
     delete[] buf;
 
@@ -163,7 +162,7 @@ bool KAKADUCodec::Decode(DataElement const &in, DataElement &out)
       bv->WriteBuffer( outfile );
       outfile.close(); // flush !
 
-      //gdcm::Filename fn( System::GetCurrentProcessFileName() );
+      //Filename fn( System::GetCurrentProcessFileName() );
       //std::string executable_path = fn.GetPath();
 #ifdef GDCM_USE_SYSTEM_KAKADU
       std::string kakadu_command = GDCM_KAKADU_EXPAND_EXECUTABLE;
@@ -182,10 +181,10 @@ bool KAKADUCodec::Decode(DataElement const &in, DataElement &out)
       int ret = system(kakadu_command.c_str());
       //std::cerr << "system: " << ret << std::endl;
 
-      size_t len = gdcm::System::FileSize(output.c_str());
+      size_t len = System::FileSize(output.c_str());
       if(!len) return false;
 
-      std::ifstream is(output.c_str());
+      std::ifstream is(output.c_str(), std::ios::binary);
       char * buf = new char[len];
       is.read(buf, len);
       os.write(buf, len);
@@ -205,7 +204,7 @@ bool KAKADUCodec::Decode(DataElement const &in, DataElement &out)
       }
     std::string str = os.str();
     assert( str.size() );
-    out.SetTag( gdcm::Tag(0x7fe0,0x0010) );
+    out.SetTag( Tag(0x7fe0,0x0010) );
     out.SetByteValue( &str[0], str.size() );
     }
   else
@@ -235,4 +234,9 @@ bool KAKADUCodec::Code(DataElement const &in, DataElement &out)
 #endif
 }
 
+ImageCodec * KAKADUCodec::Clone() const
+{
+  return nullptr;
+}
+
 } // end namespace gdcm
