@@ -1,9 +1,8 @@
 /*=========================================================================
 
   Program: GDCM (Grassroots DICOM). A DICOM library
-  Module:  $URL$
 
-  Copyright (c) 2006-2010 Mathieu Malaterre
+  Copyright (c) 2006-2011 Mathieu Malaterre
   All rights reserved.
   See Copyright.txt or http://gdcm.sourceforge.net/Copyright.html for details.
 
@@ -19,12 +18,10 @@ namespace gdcm
 {
 
 PDFCodec::PDFCodec()
-{
-}
+= default;
 
 PDFCodec::~PDFCodec()
-{
-}
+= default;
 
 bool PDFCodec::Decode(DataElement const &is, DataElement &os)
 {
