@@ -170,7 +170,7 @@ bool PixmapReader::Read()
 }
 
 // PICKER-16-MONO2-Nested_icon.dcm
-void DoIconImage(const DataSet& rootds, Pixmap& image)
+static void DoIconImage(const DataSet& rootds, Pixmap& image)
 {
   const Tag ticonimage(0x0088,0x0200);
   IconImage &pixeldata = image.GetIconImage();
@@ -354,7 +354,7 @@ void DoIconImage(const DataSet& rootds, Pixmap& image)
 }
 
 // GE_DLX-8-MONO2-Multiframe.dcm
-void DoCurves(const DataSet& ds, Pixmap& pixeldata)
+static void DoCurves(const DataSet& ds, Pixmap& pixeldata)
 {
   unsigned int numcurves;
   if( (numcurves = Curve::GetNumberOfCurves( ds )) )
@@ -406,7 +406,7 @@ void DoCurves(const DataSet& ds, Pixmap& pixeldata)
     }
 }
 
-unsigned int GetNumberOfOverlaysInternal(DataSet const & ds, std::vector<uint16_t> & overlaylist)
+static unsigned int GetNumberOfOverlaysInternal(DataSet const & ds, std::vector<uint16_t> & overlaylist)
 {
   Tag overlay(0x6000,0x0000); // First possible overlay
   bool finished = false;
@@ -491,7 +491,7 @@ unsigned int GetNumberOfOverlaysInternal(DataSet const & ds, std::vector<uint16_
   return numoverlays;
 }
 
-bool DoOverlays(const DataSet& ds, Pixmap& pixeldata)
+static bool DoOverlays(const DataSet& ds, Pixmap& pixeldata)
 {
   unsigned int numoverlays;
   std::vector<uint16_t> overlaylist;
@@ -530,11 +530,11 @@ bool DoOverlays(const DataSet& ds, Pixmap& pixeldata)
         //assert( unpack.str().size() / 8 == ((ov.GetRows() * ov.GetColumns()) + 7 ) / 8 );
         assert( ov.IsInPixelData( ) == false );
         }
-      else
+      else if( pixeldata.GetPixelFormat().GetSamplesPerPixel() == 1 )
         {
+        assert( ov.IsEmpty() );
         gdcmDebugMacro( "This image does not contains Overlay in the 0x60xx tags. "
-          << "Instead the overlay is stored in the unused bit of the Pixel Data. "
-          << "This is not supported right now"
+          << "Instead the overlay is stored in the unused bit of the Pixel Data."
           << std::endl );
         ov.IsInPixelData( true );
         // make sure Overlay is valid
@@ -551,6 +551,14 @@ bool DoOverlays(const DataSet& ds, Pixmap& pixeldata)
           }
         updateoverlayinfo[idxoverlays] = true;
         }
+      else
+        {
+        // http://dicom.nema.org/medical/dicom/current/output/chtml/part03/sect_C.9.2.html
+	// Overlay data stored in unused bit planes of Pixel Data (7FE0,0010)
+	// with Samples Per Pixel (0028,0002) of 1 was previously described in
+	// DICOM. 
+	gdcmWarningMacro( "Overlay was found, while PI is: " << pixeldata.GetPhotometricInterpretation() << " skipping.");
+        }
       }
     //std::cout << "Num of Overlays: " << numoverlays << std::endl;
     }
@@ -573,6 +581,13 @@ bool DoOverlays(const DataSet& ds, Pixmap& pixeldata)
         gdcmWarningMacro( "Invalid BitPosition: " << obp << " for overlay #" <<
           ov << " removing it." );
         }
+      else if( obp > pf.GetBitsAllocated() )
+        {
+        pixeldata.RemoveOverlay( ov );
+        updateoverlayinfo.erase( updateoverlayinfo.begin() + ov );
+        gdcmWarningMacro( "Invalid BitPosition: " << obp << " for overlay #" <<
+          ov << " removing it." );
+        }
       }
     }
 
@@ -616,7 +631,8 @@ bool PixmapReader::ReadImageInternal(MediaStorage const &ms, bool handlepixeldat
     isacrnema = true;
     const char *str = ds.GetDataElement( trecognitioncode ).GetByteValue()->GetPointer();
     assert( strncmp( str, "ACR-NEMA", strlen( "ACR-NEMA" ) ) == 0 ||
-      strncmp( str, "ACRNEMA", strlen( "ACRNEMA" ) ) == 0 );
+      strncmp( str, "ACRNEMA", strlen( "ACRNEMA" ) ) == 0 ||
+      strncmp( str, "MIPS 2.0", strlen( "MIPS 2.0" ) ) == 0 );
     (void)str;//warning removal
     }
 
@@ -991,6 +1007,17 @@ bool PixmapReader::ReadImageInternal(MediaStorage const &ms, bool handlepixeldat
       bool need = PixelData->GetTransferSyntax() == TransferSyntax::ImplicitVRBigEndianPrivateGE;
       PixelData->SetNeedByteSwap( need );
       PixelData->SetDataElement( xde );
+      if( PixelData->GetTransferSyntax().IsEncapsulated() && PixelData->GetDataElement().GetByteValue() )
+      {
+        // Pixel Data attribute is not encapsulated, let's check for simple user error
+        const ByteValue *bv = PixelData->GetDataElement().GetByteValue();
+        if( bv->GetLength() == PixelData->GetBufferLength() ||
+            bv->GetLength() == PixelData->GetBufferLength() + 1 )
+        {
+          gdcmWarningMacro( "Pixel Data was found to be raw. Fixing invalid Transfer Syntax: " << PixelData->GetTransferSyntax() );
+          PixelData->SetTransferSyntax( TransferSyntax::ExplicitVRLittleEndian );
+        }
+      }
 
       // FIXME:
       // We should check that when PixelData is RAW that Col * Dim == PixelData->GetLength()
@@ -1032,7 +1059,7 @@ bool PixmapReader::ReadImageInternal(MediaStorage const &ms, bool handlepixeldat
           assert( jpeg.GetDimensions()[1] );
           v[0] = jpeg.GetDimensions()[0];
           v[1] = jpeg.GetDimensions()[1];
-          PixelData->SetDimensions( &v[0] );
+          PixelData->SetDimensions( v.data() );
           //PixelData->SetPixelFormat( jpeg.GetPixelFormat() ); // need to handle carefully
           if( PixelData->GetPixelFormat().GetSamplesPerPixel() != jpeg.GetPixelFormat().GetSamplesPerPixel() )
           {
@@ -1081,7 +1108,11 @@ bool PixmapReader::ReadImageInternal(MediaStorage const &ms, bool handlepixeldat
   // Two cases:
   // - DataSet did not specify the lossyflag
   // - DataSet specify it to be 0, but there is still a chance it could be wrong:
-  if( !haslossyflag || !lossyflag )
+  // execute computation of lossy flag eny time the TS is encapsulated so as to
+  // update the correct PixelFormat as early as possible and not during
+  // decompression in case of mismatch:
+  if( (!haslossyflag || !lossyflag)
+   || PixelData->GetTransferSyntax().IsEncapsulated() )
     {
     PixelData->ComputeLossyFlag();
     if( PixelData->IsLossy() && (!lossyflag && haslossyflag ) )
@@ -1089,6 +1120,8 @@ bool PixmapReader::ReadImageInternal(MediaStorage const &ms, bool handlepixeldat
       // We always prefer the setting from the stream...
       gdcmWarningMacro( "DataSet set LossyFlag to 0, while Codec made the stream lossy" );
       }
+    // Make sure to combine DICOM info + pixel data bitstream:
+    PixelData->SetLossyFlag( PixelData->IsLossy() || lossyflag);
     }
 
   return true;
@@ -1107,10 +1140,21 @@ bool PixmapReader::ReadACRNEMAImage()
   if( ds.FindDataElement( timagedimensions ) )
     {
     const DataElement& de0 = ds.GetDataElement( timagedimensions );
+    unsigned short imagedimensions = 0;
+    if( de0.GetVR() == VR::SS )
+    {
+    // Data/SIEMENS_MAGNETOM-12-MONO2-Uncompressed.dcm 0028,0005 is SS
+    Element<VR::SS,VM::VM1> el0 = { 0 };
+    el0.SetFromDataElement( de0 );
+    imagedimensions = el0.GetValue();
+    }
+    else
+    {
     Attribute<0x0028,0x0005> at0 = { 0 };
     at0.SetFromDataElement( de0 );
     assert( at0.GetNumberOfValues() == 1 );
-    unsigned short imagedimensions = at0.GetValue();
+    imagedimensions = at0.GetValue();
+    }
     //assert( imagedimensions == ReadSSFromTag( timagedimensions, ss, conversion ) );
     if ( imagedimensions == 3 )
       {
@@ -1234,6 +1278,7 @@ bool PixmapReader::ReadACRNEMAImage()
 
   // 4. Do the Curves/Overlays if any
   DoCurves(ds, *PixelData);
+  // Pay attention that pf.GetSamplesPerPixel() may equal 1 (eg. LIBIDO-24-ACR_NEMA-Rectangle.dcm)
   DoOverlays(ds, *PixelData);
 
   // 5. Do the PixelData
