@@ -12,11 +12,11 @@
 
 =========================================================================*/
 #include "gdcmRescaler.h"
-#include <limits>
 #include <algorithm> // std::max
-#include <stdlib.h> // abort
-#include <string.h> // memcpy
 #include <cmath> // std::lround
+#include <cstdlib> // abort
+#include <cstring> // memcpy
+#include <limits>
 
 namespace gdcm
 {
@@ -125,8 +125,11 @@ static inline PixelFormat::ScalarType ComputeBestFit(const PixelFormat &pf, doub
   PixelFormat::ScalarType st = PixelFormat::UNKNOWN;
   assert( slope == (int)slope && intercept == (int)intercept);
 
-  const double min = slope * (double)pf.GetMin() + intercept;
-  const double max = slope * (double)pf.GetMax() + intercept;
+  assert( pf.GetMin() <= pf.GetMax() );
+  const double pfmin = slope >= 0. ? (double)pf.GetMin() : (double)pf.GetMax();
+  const double pfmax = slope >= 0. ? (double)pf.GetMax() : (double)pf.GetMin();
+  const double min = slope * pfmin + intercept;
+  const double max = slope * pfmax + intercept;
   assert( min <= max );
   assert( min == (int64_t)min && max == (int64_t)max );
   if( min >= 0 ) // unsigned
@@ -143,6 +146,11 @@ static inline PixelFormat::ScalarType ComputeBestFit(const PixelFormat &pf, doub
       {
       st = PixelFormat::UINT32;
       }
+    else if( max <= static_cast<double>(std::numeric_limits<uint64_t>::max()) )
+      {
+      // very large value in Rescale Slope ?
+      return PixelFormat::FLOAT64;
+      }
     else
       {
       gdcmErrorMacro( "Unhandled Pixel Format" );
@@ -166,6 +174,12 @@ static inline PixelFormat::ScalarType ComputeBestFit(const PixelFormat &pf, doub
       {
       st = PixelFormat::INT32;
       }
+    else if( max <= static_cast<double>(std::numeric_limits<int64_t>::max())
+      && min >= static_cast<double>(std::numeric_limits<int64_t>::min() ) )
+      {
+      // very large value in Rescale Slope ?
+      return PixelFormat::FLOAT64;
+      }
     else
       {
       gdcmErrorMacro( "Unhandled Pixel Format" );
@@ -182,6 +196,11 @@ static inline PixelFormat::ScalarType ComputeBestFit(const PixelFormat &pf, doub
 PixelFormat::ScalarType Rescaler::ComputeInterceptSlopePixelType()
 {
   assert( PF != PixelFormat::UNKNOWN );
+  if( PF.GetSamplesPerPixel() != 1 )
+    {
+    gdcmErrorMacro( "Sample Per Pixel is required to be 1" );
+    return PF;
+    }
   PixelFormat::ScalarType output = PixelFormat::UNKNOWN;
   if( PF == PixelFormat::SINGLEBIT ) return PixelFormat::SINGLEBIT;
   if( Slope != (int)Slope || Intercept != (int)Intercept)
@@ -402,11 +421,17 @@ static PixelFormat ComputeInverseBestFitFromMinMax(/*const PixelFormat &pf,*/ do
   PixelFormat st = PixelFormat::UNKNOWN;
   //assert( slope == (int)slope && intercept == (int)intercept);
 
+  assert( _min <= _max );
   double dmin = (_min - intercept ) / slope;
   double dmax = (_max - intercept ) / slope;
+  if( slope < 0 )
+    {
+    dmin = (_max - intercept ) / slope;
+    dmax = (_min - intercept ) / slope;
+    }
   assert( dmin <= dmax );
-  assert( dmax <= std::numeric_limits<int64_t>::max() );
-  assert( dmin >= std::numeric_limits<int64_t>::min() );
+  assert( dmax <= static_cast<double>(std::numeric_limits<int64_t>::max() ) );
+  assert( dmin >= static_cast<double>(std::numeric_limits<int64_t>::min() ) );
   /*
    * Tricky: what happen in the case where floating point approximate dmax as: 65535.000244081035
    * Take for instance: _max = 64527, intercept = -1024, slope = 1.000244140625
@@ -486,6 +511,21 @@ static PixelFormat ComputeInverseBestFitFromMinMax(/*const PixelFormat &pf,*/ do
   return st;
 }
 
+void Rescaler::SetMinMaxForPixelType(double min, double max)
+{
+  if( min < max )
+  {
+    ScalarRangeMin = min;
+    ScalarRangeMax = max;
+  }
+  else
+  {
+    gdcmWarningMacro( "Min > Max. Correcting" );
+    ScalarRangeMin = max;
+    ScalarRangeMax = min;
+  }
+}
+
 PixelFormat Rescaler::ComputePixelTypeFromMinMax()
 {
   assert( PF != PixelFormat::UNKNOWN );
