@@ -1,6 +1,6 @@
 /*=========================================================================
  *
- *  Copyright Insight Software Consortium
+ *  Copyright NumFOCUS
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -45,7 +45,7 @@ public:
 namespace network{
 
 std::vector<PresentationDataValue> CStoreRQ::ConstructPDV(
-const ULConnection &inConnection, File const & file )
+const ULConnection &inConnection, File const & file, bool writeDataSet /*=true*/ )
 {
 const DataSet* inDataSet = &file.GetDataSet();
 
@@ -64,7 +64,7 @@ const DataSet* inDataSet = &file.GetDataSet();
   MediaStorage mst;
   if (!mst.SetFromDataSet(*inDataSet))
     {
-    assert( 0 );
+    throw Exception("Missing MediaStorage");
     }
   UIDs uid;
   uid.SetFromUID( MediaStorage::GetMSString(mst) );
@@ -120,12 +120,12 @@ const DataSet* inDataSet = &file.GetDataSet();
   // the following make sure that the accepted Presentation Context match the actual encoding
   // of the current File
   // ADV: technically we could use an explicit VR encoded dataset and send it over
-  // an implicit TS accecpted Transfer syntax. However thing do not interchange well
+  // an implicit TS accepted Transfer syntax. However thing do not interchange well
   // so we really need a filter to check whether conversion is ok or not.
-  if( acpc == 0 )
+  if( acpc == nullptr )
     {
     // Technically we should fallback to something else. Anyway lets' give up
-    // and hope the use will convert the encapsulated stream to something else...
+    // and hope the user will convert the encapsulated stream to something else...
     throw Exception("Server side refuse our proposed PC.");
     }
 
@@ -213,6 +213,8 @@ static uint32_t messageid = 1;
   thePDVs.push_back(thePDV);
 }
 
+if( !writeDataSet )
+	return thePDVs ;
   // now let's chunk'ate the dataset:
 {
   std::stringstream ss;
@@ -244,13 +246,16 @@ static uint32_t messageid = 1;
     thePDV.SetPresentationContextID( prescontid );
 
     thePDV.SetBlob( sub );
-    if( remaining == maxpdu )
+    cur += remaining;
+
+    if( cur < end )
       thePDV.SetMessageHeader( 0 );
     else
+      {
+      assert( cur == end );
       thePDV.SetMessageHeader( 2 );
+      }
     thePDVs.push_back(thePDV);
-
-    cur += remaining;
     }
 }
 
