@@ -1,6 +1,6 @@
 /*=========================================================================
  *
- *  Copyright Insight Software Consortium
+ *  Copyright NumFOCUS
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -158,10 +158,10 @@ EStateID ULActionAE6::PerformAction(Subject *, ULEvent& inEvent, ULConnection& i
     acceptable = false; //can't accept an empty set of pdus.
     //also, requrie little endian, not sure how to set that, but it should be here.
   }
-  AAssociateRQPDU* rqpdu;
+  AAssociateRQPDU* rqpdu = nullptr;
   if (acceptable){
     rqpdu = dynamic_cast<AAssociateRQPDU*>(inEvent.GetPDUs()[0]);
-    if (rqpdu == NULL){
+    if (rqpdu == nullptr){
       acceptable = false;
     }
   }
@@ -175,6 +175,7 @@ EStateID ULActionAE6::PerformAction(Subject *, ULEvent& inEvent, ULConnection& i
 
     AAssociateACPDU acpdu;
 
+    assert( rqpdu->GetNumberOfPresentationContext() );
     for( unsigned int index = 0; index < rqpdu->GetNumberOfPresentationContext(); index++ )
       {
       // FIXME / HARDCODED We only ever accept Little Endian
@@ -187,26 +188,48 @@ EStateID ULActionAE6::PerformAction(Subject *, ULEvent& inEvent, ULConnection& i
       // BOGUS (MM):
       //inConnection.AddAcceptedPresentationContext(pc);
 
-      uint8_t id = pc.GetPresentationContextID();
+      const uint8_t id = pc.GetPresentationContextID();
 
       std::vector<TransferSyntaxSub> const & tsSet = pc.GetTransferSyntaxes();
       std::vector<TransferSyntaxSub>::const_iterator tsitor;
-      bool hasLittleEndian = false;
+      // PS 3.8 Table 9-18 PRESENTATION CONTEXT ITEM FIELDS
+      uint8_t result = 4; // transfer-syntaxes-not-supported (provider rejection)
       for (tsitor = tsSet.begin(); tsitor < tsSet.end(); tsitor++)
         {
-        if (strcmp(tsitor->GetName(), ts1.GetName())==0)
+        //gdcmDebugMacro( "Checking: [" << tsitor->GetName() << "] vs [" << ts1.GetName() << "]" << std::endl );
+        if (strcmp(tsitor->GetName(), ts1.GetName()) == 0 )
           {
-          hasLittleEndian = true;
+          result = 0; // 0 - acceptance
+          inConnection.SetCStoreTransferSyntax( ts1 );
+          pcac1.SetTransferSyntax( ts1 );
           }
         }
-      if (!hasLittleEndian) continue; //don't add this presentation context, because
-      //the client doesn't know how to handle it.
-
+      if( result )
+        {
+        gdcmWarningMacro( "Could not find Implicit or Explicit Little Endian in Response. Giving another try" );
+        // Okay little endian implicit was not found, this happen sometimes, for eg with DVTk, let's be nice and accept also Explicit
+        TransferSyntaxSub ts2;
+        ts2.SetNameFromUID( UIDs::ExplicitVRLittleEndian );
+        for (tsitor = tsSet.begin(); tsitor < tsSet.end(); tsitor++)
+          {
+          //gdcmDebugMacro( "Checking: [" << tsitor->GetName() << "] vs [" << ts1.GetName() << "]" << std::endl );
+          if (strcmp(tsitor->GetName(), ts2.GetName()) == 0 )
+            {
+            result = 0; // 0 - acceptance
+            inConnection.SetCStoreTransferSyntax( ts2 );
+            pcac1.SetTransferSyntax( ts2 );
+            }
+          }
+        }
+      if( result )
+        {
+        gdcmErrorMacro( "Could not find Implicit or Explicit Little Endian in Response. Giving up" );
+        }
       pcac1.SetPresentationContextID( id );
-      pcac1.SetTransferSyntax( ts1 );
-      //pcac1.SetTransferSyntax( tsSet[0] );
+      pcac1.SetReason( result );
       acpdu.AddPresentationContextAC( pcac1 );
     }
+    assert( acpdu.GetNumberOfPresentationContextAC() );
 
     // Init AE-Titles:
     acpdu.InitFromRQ( *rqpdu );
