@@ -28,7 +28,7 @@ class Foo : public Object {
 
 class Containter {
 public:
-  Containter():Instance(0) {}
+  Containter():Instance(nullptr) {}
   SmartPointer<Object> Instance;
 };
 
@@ -40,7 +40,7 @@ void Fill(SmartPointer<Foo> &p)
   p = &rp;
 }
 
-SmartPointer<Foo> gf;
+static SmartPointer<Foo> gf;
 
 SmartPointer<Foo> TestReturn(int i)
 {
@@ -64,7 +64,7 @@ SmartPointer<Foo> TestReturn(int i)
     {
     return new Foo;
     }
-  return 0;
+  return nullptr;
 }
 
 //class Object2 : public Foo {};
@@ -111,7 +111,10 @@ int TestSmartPointer(int, char *[])
   for(int i = 0; i < 5; ++i)
     {
     SmartPointer<Foo> f = TestReturn(i);
-    f->foo();
+    if( f )
+      {
+      f->foo();
+      }
     }
 
   return 0;
