@@ -125,6 +125,7 @@ int TestOperator()
 }
 
 int TestTag(int , char * [])
+{
 {
   const uint32_t dummy = 0x12345678;
   gdcm::Tag t16(0x1234, 0x5678);
@@ -168,7 +169,7 @@ int TestTag(int , char * [])
   std::cout << " stored in RAM as :";
   for (i=0;i<sizeof(uint32_t);i++)
     {
-    std::cout << std::hex <<"[" <<(uint32_t)((uint8_t*)&tag)[i] << "] " ;
+    std::cout << std::hex <<"[" <<(uint32_t)((const uint8_t*)&tag)[i] << "] " ;
     }
   std::cout << std::endl;
 
@@ -179,11 +180,11 @@ int TestTag(int , char * [])
   std::cout << " stored in RAM as : ";
   for (i=0;i<sizeof(uint16_t);i++)
     {
-    std::cout << std::hex <<"[" <<(uint32_t)((uint8_t*)&group)[i] << "] " ;
+    std::cout << std::hex <<"[" <<(uint32_t)((const uint8_t*)&group)[i] << "] " ;
     }
   for (i=0;i<sizeof(uint16_t);i++)
     {
-    std::cout << std::hex <<"[" <<(uint32_t)((uint8_t*)&element)[i] << "] " ;
+    std::cout << std::hex <<"[" <<(uint32_t)((const uint8_t*)&element)[i] << "] " ;
     }
   std::cout << std::endl;
 
@@ -308,6 +309,7 @@ int TestTag(int , char * [])
     ||!(O2 < O4)
     ||!(O3 < O4) )
     {std::cout << "31" << std::endl ; return 1; }
+}
 
   // IsGroupXX:
     {
