@@ -69,12 +69,12 @@ int TestWrite(const char *subdir, const char* filename, bool recursing, bool ver
   char digest[33], outdigest[33];
   Testing::ComputeFileMD5(filename, digest);
   Testing::ComputeFileMD5(outfilename.c_str(), outdigest);
-  if( strcmp(digest, outdigest) )
+  if( strcmp(digest, outdigest) != 0 )
     {
     if (recursing)
       return 1;
     // too bad the file is not identical, so let's be paranoid and
-    // try to reread-rewrite this just-writen file:
+    // try to reread-rewrite this just-written file:
     // TODO: Copy file System::CopyFile( );
     std::string subsubdir = subdir;
     subsubdir += "/";
@@ -90,14 +90,14 @@ int TestWrite(const char *subdir, const char* filename, bool recursing, bool ver
      {
      if( strcmp(ref, outdigest) == 0 )
        {
-       // ok this situation was already analyzed and the writen file is
+       // ok this situation was already analyzed and the written file is
        // readable by dcmtk and such
        //size_t size1 = System::FileSize( filename );
        //size_t size2 = System::FileSize( outfilename.c_str() );
        //assert( size1 == size2 ); // cannot deal with implicit VR meta data header
        return 0;
        }
-      std::cerr << "incompatible ref:" << ref << " vs " << outdigest << " for file: " << filename << " & " << outfilename << std::endl;
+      std::cerr << "incompatible ref: " << ref << " vs " << outdigest << " for file: " << filename << " & " << outfilename << std::endl;
       return 1; // ref exist but does not match, how is that possible ?
      }
    //if( !ref )
@@ -116,7 +116,7 @@ int TestWrite(const char *subdir, const char* filename, bool recursing, bool ver
     size_t size2 = System::FileSize( outfilename.c_str() );
     if( size1 != size2 ) return 1;
     if(verbose)
-    std::cerr << filename << " and " << outfilename << " are identical\n";
+      std::cerr << filename << " and " << outfilename << " are identical\n";
     return 0;
     }
 }
