@@ -16,7 +16,7 @@
 #include "gdcmDataElement.h"
 #include "gdcmSequenceOfItems.h"
 
-int TestDataElementValueAsSQ(int argc, char *argv[])
+int TestDataElementValueAsSQ(int , char *[])
 {
   int ret = 0;
   const char *filenames[] = {
@@ -31,7 +31,7 @@ int TestDataElementValueAsSQ(int argc, char *argv[])
   const char *root = gdcm::Testing::GetDataRoot();
   if( !root || !*root )
     {
-    std::cerr << "root is not defiend" << std::endl;
+    std::cerr << "root is not defined" << std::endl;
     return 1;
     }
   std::string sroot = root;
