@@ -25,6 +25,7 @@
 #include <vtkXMLPolyDataWriter.h>
 #include <vtkRenderWindowInteractor.h>
 #include <vtkImageColorViewer.h>
+#include "vtkVersion.h"
 
 #include "gdcmReader.h"
 #include "gdcmAttribute.h"
@@ -156,23 +157,38 @@ int main(int argc, char *argv[])
     std::cout << at4.GetValue(0) << std::endl;
 
     vtkDoubleArray *d = vtkDoubleArray::New();
-    d->SetArray( (double*)pts , at1.GetValue() * at2.GetValue() , 0 );
+    d->SetArray( const_cast<double*>(pts) , at1.GetValue() * at2.GetValue() , 0 );
 
     vtkImageData *img = vtkImageData::New();
     img->Initialize();
     img->SetDimensions( at2.GetValue(), at1.GetValue(), 1 );
     //imgb->SetExtent(1, xdim, 1, ydim, 1, zdim);
+#if (VTK_MAJOR_VERSION >= 6)
+    assert(0);
+#else
     img->SetScalarTypeToDouble();
+#endif
     img->SetSpacing( at3.GetValue(1), at3.GetValue(0), 1); // FIXME image is upside down
     img->SetOrigin( at4.GetValue(0), at4.GetValue(1), 1);
+#if (VTK_MAJOR_VERSION >= 6)
+    assert(0);
+#else
     img->SetNumberOfScalarComponents(1);
+#endif
     img->GetPointData()->SetScalars(d);
 
+#if (VTK_MAJOR_VERSION >= 6)
+#else
     img->Update();
+#endif
     img->Print(std::cout);
 
     vtkXMLImageDataWriter *writeb= vtkXMLImageDataWriter::New();
+#if (VTK_MAJOR_VERSION >= 6)
+    writeb->SetInputData( img );
+#else
     writeb->SetInput( img );
+#endif
     writeb->SetFileName( outfilename );
     writeb->Write( );
 /*
@@ -235,8 +251,8 @@ int main(int argc, char *argv[])
     for(unsigned int i = 0; i < npts; ++i)
       {
       float x[3] = {};
-      x[0] = ptr[2*i+0];
-      x[1] = ptr[2*i+1];
+      x[0] = (float)ptr[2*i+0];
+      x[1] = (float)ptr[2*i+1];
       //x[2] = pts[i+2];
       vtkIdType ptId = newPts->InsertNextPoint( x );
       //std::cout << x[0] << "," << x[1] << "," << x[2] << std::endl;
@@ -252,7 +268,10 @@ int main(int argc, char *argv[])
     polys->Delete();
     //output->GetCellData()->SetScalars(scalars);
     //scalars->Delete();
+#if (VTK_MAJOR_VERSION >= 6)
+#else
     output->Update();
+#endif
     output->Print( std::cout );
 
 
@@ -263,7 +282,11 @@ int main(int argc, char *argv[])
    vtkRenderWindowInteractor *iren = vtkRenderWindowInteractor::New();
 
    vtkImageColorViewer *viewer = vtkImageColorViewer::New();
+#if (VTK_MAJOR_VERSION >= 6)
+   viewer->SetInputData(img);
+#else
    viewer->SetInput(img);
+#endif
    viewer->SetupInteractor(iren);
    viewer->SetSize(600, 600);
    viewer->GetRenderer()->ResetCameraClippingRange();
@@ -272,7 +295,11 @@ int main(int argc, char *argv[])
 
   vtkPolyDataMapper *cubeMapper = vtkPolyDataMapper::New();
   //vtkPolyDataMapper2D* cubeMapper = vtkPolyDataMapper2D::New();
+#if (VTK_MAJOR_VERSION >= 6)
+      cubeMapper->SetInputData( output );
+#else
       cubeMapper->SetInput( output );
+#endif
       cubeMapper->SetScalarRange(0,7);
   vtkActor *cubeActor = vtkActor::New();
   //vtkActor2D* cubeActor = vtkActor2D::New();
@@ -283,7 +310,11 @@ int main(int argc, char *argv[])
 viewer->GetRenderer()->AddActor( cubeActor );
 
     vtkXMLPolyDataWriter *writec= vtkXMLPolyDataWriter::New();
+#if (VTK_MAJOR_VERSION >= 6)
+    writec->SetInputData( output );
+#else
     writec->SetInput( output );
+#endif
     writec->SetFileName( outfilename2 );
     writec->Write( );
 
