@@ -11,23 +11,28 @@
      PURPOSE.  See the above copyright notice for more information.
 
 =========================================================================*/
+#include "vtkVersion.h"
 #include "vtkGDCMImageReader.h"
 #include "vtkPiecewiseFunction.h"
 #include "vtkColorTransferFunction.h"
 #include "vtkVolume.h"
 #include "vtkVolumeProperty.h"
+#if VTK_MAJOR_VERSION < 7
 #include "vtkVolumeTextureMapper3D.h"
+#endif
 #include "vtkFixedPointVolumeRayCastMapper.h"
 #include "vtkInteractorStyleTrackballCamera.h"
 #include "vtkRenderer.h"
 #include "vtkRenderWindow.h"
 #include "vtkImageClip.h"
 #include "vtkRenderWindowInteractor.h"
+#include "vtkVersion.h"
 
 
 // gdcmvolume gdcmData/GE_DLX-8-MONO2-Multiframe-Jpeg_Lossless.dcm
 int main(int argc, char *argv[])
 {
+  if( argc < 2 ) return 1;
   vtkGDCMImageReader *reader = vtkGDCMImageReader::New();
   reader->SetFileName( argv[1] );
   reader->Update();
