@@ -14,6 +14,7 @@
 #include "vtkGDCMThreadedImageReader2.h"
 #include "vtkGDCMImageReader.h"
 #include "vtkCommand.h"
+#include "vtkVersion.h"
 
 #include "gdcmDirectory.h"
 #include "gdcmSystem.h"
@@ -71,7 +72,7 @@ int ExecuteInformation(const char *filename, TReader *vtkreader)
     if ( filenames && filenames->GetNumberOfValues() > 1 )
       {
       dataextent[4] = 0;
-      dataextent[5] = filenames->GetNumberOfValues() - 1;
+      dataextent[5] = (int)filenames->GetNumberOfValues() - 1;
       }
     else
       {
@@ -150,7 +151,7 @@ int TestvtkGDCMThreadedImageRead2(const char *filename, bool verbose = false)
     gdcm::Directory d;
     d.Load( filename );
     gdcm::Directory::FilenamesType l = d.GetFilenames();
-    const unsigned int nfiles = l.size();
+    const size_t nfiles = l.size();
     vtkStringArray *sarray = vtkStringArray::New();
     for(unsigned int i = 0; i < nfiles; ++i)
       {
@@ -167,7 +168,7 @@ int TestvtkGDCMThreadedImageRead2(const char *filename, bool verbose = false)
     refimage = filename;
     }
 
-  // In all cases we need to explicitely say what the image type is:
+  // In all cases we need to explicitly say what the image type is:
   if( !ExecuteInformation<TReader>(refimage, reader) )
     {
     std::cerr << "file: " << refimage << " is not an image. giving up" << std::endl;
@@ -191,7 +192,11 @@ int TestvtkGDCMThreadedImageRead2(const char *filename, bool verbose = false)
   if( reader->GetNumberOfOverlays() )
     {
     vtkPNGWriter *writer = vtkPNGWriter::New();
+#if (VTK_MAJOR_VERSION >= 6)
+    writer->SetInputConnection( reader->GetOutputPort(1) );
+#else
     writer->SetInput( reader->GetOutput(1) );
+#endif
     const char subdir[] = "TestvtkGDCMThreadedImageReader2";
     // Create directory first:
     std::string tmpdir = gdcm::Testing::GetTempDirectory( subdir );
