@@ -1,9 +1,8 @@
 /*=========================================================================
 
   Program: GDCM (Grassroots DICOM). A DICOM library
-  Module:  $URL$
 
-  Copyright (c) 2006-2009 Mathieu Malaterre
+  Copyright (c) 2006-2011 Mathieu Malaterre
   All rights reserved.
   See Copyright.txt or http://gdcm.sourceforge.net/Copyright.html for details.
 
@@ -12,8 +11,8 @@
      PURPOSE.  See the above copyright notice for more information.
 
 =========================================================================*/
-#ifndef __gdcm_ljpeg16_h
-#define __gdcm_ljpeg16_h
+#ifndef GDCM_LJPEG16_H
+#define GDCM_LJPEG16_H
 
 /* Use the ljpeg library configured for gdcm.  */
 #include "gdcmTypes.h"
