@@ -1,9 +1,8 @@
 /*=========================================================================
 
   Program: GDCM (Grassroots DICOM). A DICOM library
-  Module:  $URL$
 
-  Copyright (c) 2006-2010 Mathieu Malaterre
+  Copyright (c) 2006-2011 Mathieu Malaterre
   All rights reserved.
   See Copyright.txt or http://gdcm.sourceforge.net/Copyright.html for details.
 
@@ -19,25 +18,8 @@
 #include "gdcmTypes.h"
 #ifdef GDCM_USE_SYSTEM_OPENJPEG
 #include <openjpeg.h>
-// MM:
-// See openjpeg issue #3:
-// http://code.google.com/p/openjpeg/issues/detail?id=3
-//#include <j2k.h>
-//#include <jp2.h>
-
-// Instead duplicate header (I know this is bad)
-extern "C" {
-#include "gdcm_j2k.h"
-#include "gdcm_jp2.h"
-}
-
 #else
-extern "C" {
-#include <gdcmopenjpeg/libopenjpeg/openjpeg.h>
-#include <gdcmopenjpeg/libopenjpeg/j2k.h>
-#include <gdcmopenjpeg/libopenjpeg/jp2.h>
-}
+#include "gdcmopenjpeg/src/lib/openjp2/openjpeg.h"
 #endif
 
 #endif
-
